#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_meraki_auth_users
short_description: Resource module for networks _meraki _auth _users
description:
  - Manage operations create, update and delete of the resource networks _meraki _auth
    _users. - > Authorize a user configured with Meraki Authentication for a network
    currently supports 802.1X, splash guest, and client VPN users, and currently,
    organizations have a 50,000 user cap .
  - Delete an 802.1X RADIUS user, or deauthorize and optionally delete a splash guest
    or client VPN user. - > Update a user configured with Meraki Authentication currently,
    802.1X RADIUS, splash guest, and client VPN users can be updated .
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  accountType:
    description: Authorization type for user. Can be 'Guest' or '802.1X' for wireless
      networks, or 'Client VPN' for MX networks. Defaults to '802.1X'.
    type: str
  authorizations:
    description: Authorization zones and expiration dates for the user.
    elements: dict
    suboptions:
      expiresAt:
        description: Date for authorization to expire. Set to 'Never' for the authorization
          to not expire, which is the default.
        type: str
      ssidNumber:
        description: Required for wireless networks. The SSID for which the user is
          being authorized, which must be configured for the user's given accountType.
        type: int
    type: list
  delete:
    description: Delete query parameter. If the ID supplied is for a splash guest
      or client VPN user, and that user is not authorized for any other networks in
      the organization, then also delete the user. 802.1X RADIUS users are always
      deleted regardless of this optional attribute.
    type: bool
  email:
    description: Email address of the user.
    type: str
  emailPasswordToUser:
    description: Whether or not Meraki should email the password to user. Default
      is false.
    type: bool
  isAdmin:
    description: Whether or not the user is a Dashboard administrator.
    type: bool
  merakiAuthUserId:
    description: MerakiAuthUserId path parameter. Meraki auth user ID.
    type: str
  name:
    description: Name of the user. Only required If the user is not a Dashboard administrator.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  password:
    description: The password for this user account. Only required If the user is
      not a Dashboard administrator.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks createNetworkMerakiAuthUser
    description: Complete reference of the createNetworkMerakiAuthUser API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-meraki-auth-user
  - name: Cisco Meraki documentation for networks deleteNetworkMerakiAuthUser
    description: Complete reference of the deleteNetworkMerakiAuthUser API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-meraki-auth-user
  - name: Cisco Meraki documentation for networks updateNetworkMerakiAuthUser
    description: Complete reference of the updateNetworkMerakiAuthUser API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-meraki-auth-user
notes:
  - SDK Method used are
    networks.Networks.create_network_meraki_auth_user,
    networks.Networks.delete_network_meraki_auth_user,
    networks.Networks.update_network_meraki_auth_user,
  - Paths used are
    post /networks/{networkId}/merakiAuthUsers,
    delete /networks/{networkId}/merakiAuthUsers/{merakiAuthUserId},
    put /networks/{networkId}/merakiAuthUsers/{merakiAuthUserId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_meraki_auth_users:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    accountType: 802.1X
    authorizations:
      - expiresAt: '2018-03-13T00:00:00.090210Z'
        ssidNumber: 1
    email: miles@meraki.com
    emailPasswordToUser: false
    isAdmin: false
    name: Miles Meraki
    networkId: string
    password: secret
- name: Delete by id
  cisco.meraki.networks_meraki_auth_users:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    delete: true
    merakiAuthUserId: string
    networkId: string
- name: Update by id
  cisco.meraki.networks_meraki_auth_users:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    authorizations:
      - expiresAt: '2018-03-13T00:00:00.090210Z'
        ssidNumber: 1
    emailPasswordToUser: false
    merakiAuthUserId: string
    name: Miles Meraki
    networkId: string
    password: secret
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "email": "string",
      "name": "string",
      "createdAt": "string",
      "accountType": "string",
      "isAdmin": true,
      "authorizations": [
        {
          "ssidNumber": 0,
          "authorizedZone": "string",
          "expiresAt": "string",
          "authorizedByName": "string",
          "authorizedByEmail": "string"
        }
      ]
    }
"""



#!/usr/bin/python

# Copyright (c) 2024, Stanislav Shamilov <shamilovstas@protonmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: decompress
short_description: Decompresses compressed files
version_added: 10.1.0
description:
  - Decompresses compressed files.
  - The source (compressed) file and destination (decompressed) files are on the remote host.
  - Source file can be deleted after decompression.
extends_documentation_fragment:
  - ansible.builtin.files
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  src:
    description:
      - Remote absolute path for the file to decompress.
    type: path
    required: true
  dest:
    description:
      - The file name of the destination file where the compressed file is decompressed.
      - If the destination file exists, it is truncated and overwritten.
      - If not specified, the destination filename is derived from O(src) by removing the compression format extension. For
        example, when O(src) is V(/path/to/file.txt.gz) and O(format) is V(gz), O(dest) is V(/path/to/file.txt). If the O(src)
        file does not have an extension for the current O(format), the O(dest) filename is made by appending C(_decompressed)
        to the O(src) filename. For instance, when O(src) is V(/path/to/file.myextension), the (dest) filename is V(/path/to/file.myextension_decompressed).
    type: path
  format:
    description:
      - The type of compression to use to decompress.
    type: str
    choices: [gz, bz2, xz]
    default: gz
  remove:
    description:
      - Remove original compressed file after decompression.
    type: bool
    default: false
requirements:
  - Requires C(lzma) (standard library of Python 3) if using C(xz) format.
author:
  - Stanislav Shamilov (@shamilovstas)
"""

EXAMPLES = r"""
- name: Decompress file /path/to/file.txt.gz into /path/to/file.txt (gz compression is used by default)
  community.general.decompress:
    src: /path/to/file.txt.gz
    dest: /path/to/file.txt

- name: Decompress file /path/to/file.txt.gz into /path/to/file.txt
  community.general.decompress:
    src: /path/to/file.txt.gz

- name: Decompress file compressed with bzip2
  community.general.decompress:
    src: /path/to/file.txt.bz2
    dest: /path/to/file.txt
    format: bz2

- name: Decompress file and delete the compressed file afterwards
  community.general.decompress:
    src: /path/to/file.txt.gz
    dest: /path/to/file.txt
    remove: true
"""

RETURN = r"""
dest:
  description: Path to decompressed file.
  type: str
  returned: success
  sample: /path/to/file.txt
"""

import bz2
import filecmp
import gzip
import os
import shutil
import tempfile

from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.general.plugins.module_utils import _deps as deps
from ansible_collections.community.general.plugins.module_utils._mh.module_helper import ModuleHelper

with deps.declare("lzma"):
    import lzma


def lzma_decompress(src):
    return lzma.open(src, "rb")


def bz2_decompress(src):
    return bz2.open(src, "rb")


def gzip_decompress(src):
    return gzip.open(src, "rb")


def decompress(b_src, b_dest, handler):
    with handler(b_src) as src_file:
        with open(b_dest, "wb") as dest_file:
            shutil.copyfileobj(src_file, dest_file)


class Decompress(ModuleHelper):
    output_params = "dest"

    module = dict(
        argument_spec=dict(
            src=dict(type="path", required=True),
            dest=dict(type="path"),
            format=dict(type="str", default="gz", choices=["gz", "bz2", "xz"]),
            remove=dict(type="bool", default=False),
        ),
        add_file_common_args=True,
        supports_check_mode=True,
    )

    def __init_module__(self):
        self.handlers = {"gz": gzip_decompress, "bz2": bz2_decompress, "xz": lzma_decompress}
        if self.vars.dest is None:
            self.vars.dest = self.get_destination_filename()
        deps.validate(self.module)
        self.configure()

    def configure(self):
        b_dest = to_bytes(self.vars.dest, errors="surrogate_or_strict")
        b_src = to_bytes(self.vars.src, errors="surrogate_or_strict")
        if not os.path.exists(b_src):
            if self.vars.remove and os.path.exists(b_dest):
                self.module.exit_json(changed=False)
            else:
                self.do_raise(msg=f"Path does not exist: '{b_src}'")
        if os.path.isdir(b_src):
            self.do_raise(msg=f"Cannot decompress directory '{b_src}'")
        if os.path.isdir(b_dest):
            self.do_raise(msg=f"Destination is a directory, cannot decompress: '{b_dest}'")

    def __run__(self):
        b_dest = to_bytes(self.vars.dest, errors="surrogate_or_strict")
        b_src = to_bytes(self.vars.src, errors="surrogate_or_strict")

        file_args = self.module.load_file_common_arguments(self.module.params, path=self.vars.dest)
        handler = self.handlers[self.vars.format]
        try:
            tempfd, temppath = tempfile.mkstemp(dir=self.module.tmpdir)
            self.module.add_cleanup_file(temppath)
            b_temppath = to_bytes(temppath, errors="surrogate_or_strict")
            decompress(b_src, b_temppath, handler)
        except OSError as e:
            self.do_raise(msg=f"Unable to create temporary file '{e}'")

        if os.path.exists(b_dest):
            self.changed = not filecmp.cmp(b_temppath, b_dest, shallow=False)
        else:
            self.changed = True

        if self.changed and not self.module.check_mode:
            try:
                self.module.atomic_move(b_temppath, b_dest)
            except OSError:
                self.do_raise(msg=f"Unable to move temporary file '{b_temppath}' to '{self.vars.dest}'")

        if self.vars.remove and not self.check_mode:
            os.remove(b_src)
        self.changed = self.module.set_fs_attributes_if_different(file_args, self.changed)

    def get_destination_filename(self):
        src = self.vars.src
        fmt_extension = f".{self.vars.format}"
        if src.endswith(fmt_extension) and len(src) > len(fmt_extension):
            filename = src[: -len(fmt_extension)]
        else:
            filename = f"{src}_decompressed"
        return filename


def main():
    Decompress.execute()


if __name__ == "__main__":
    main()



# Based on jail.py
# (c) 2013, Michael Scherer <misc@zarb.org>
# (c) 2015, Toshio Kuratomi <tkuratomi@ansible.com>
# (c) 2016, Stephan Lohse <dev-github@ploek.org>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Stephan Lohse (!UNKNOWN) <dev-github@ploek.org>
name: iocage
short_description: Run tasks in iocage jails
description:
  - Run commands or put/fetch files to an existing iocage jail.
options:
  remote_addr:
    description:
      - Path to the jail.
    type: string
    vars:
      - name: ansible_host
      - name: ansible_iocage_host
  remote_user:
    description:
      - User to execute as inside the jail.
    type: string
    vars:
      - name: ansible_user
      - name: ansible_iocage_user
"""

import subprocess

from ansible.errors import AnsibleError
from ansible.module_utils.common.text.converters import to_native
from ansible.utils.display import Display

from ansible_collections.community.general.plugins.connection.jail import Connection as Jail

display = Display()


class Connection(Jail):
    """Local iocage based connections"""

    transport = "community.general.iocage"

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        self.ioc_jail = play_context.remote_addr

        self.iocage_cmd = Jail._search_executable("iocage")

        jail_uuid = self.get_jail_uuid()

        kwargs[Jail.modified_jailname_key] = f"ioc-{jail_uuid}"

        display.vvv(
            f"Jail {self.ioc_jail} has been translated to {kwargs[Jail.modified_jailname_key]}",
            host=kwargs[Jail.modified_jailname_key],
        )

        super().__init__(play_context, new_stdin, *args, **kwargs)

    def get_jail_uuid(self):
        p = subprocess.Popen(
            [self.iocage_cmd, "get", "host_hostuuid", self.ioc_jail],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        )

        stdout, stderr = p.communicate()

        if stdout is not None:
            stdout = to_native(stdout)

        if stderr is not None:
            stderr = to_native(stderr)

        # otherwise p.returncode would not be set
        p.wait()

        if p.returncode != 0:
            raise AnsibleError(f"iocage returned an error: {stdout}")

        return stdout.strip("\n")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_login_security
short_description: Resource module for organizations _login _security
description:
  - Manage operation update of the resource organizations _login _security.
  - Update the login security settings for an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  accountLockoutAttempts:
    description: Number of consecutive failed login attempts after which users' accounts
      will be locked.
    type: int
  apiAuthentication:
    description: Details for indicating whether organization will restrict access
      to API (but not Dashboard) to certain IP addresses.
    suboptions:
      ipRestrictionsForKeys:
        description: Details for API-only IP restrictions.
        suboptions:
          enabled:
            description: Boolean indicating whether the organization will restrict
              API key (not Dashboard GUI) usage to a specific list of IP addresses
              or CIDR ranges.
            type: bool
          ranges:
            description: List of acceptable IP ranges. Entries can be single IP addresses,
              IP address ranges, and CIDR subnets.
            elements: str
            type: list
        type: dict
    type: dict
  enforceAccountLockout:
    description: Boolean indicating whether users' Dashboard accounts will be locked
      out after a specified number of consecutive failed login attempts.
    type: bool
  enforceDifferentPasswords:
    description: Boolean indicating whether users, when setting a new password, are
      forced to choose a new password that is different from any past passwords.
    type: bool
  enforceIdleTimeout:
    description: Boolean indicating whether users will be logged out after being idle
      for the specified number of minutes.
    type: bool
  enforceLoginIpRanges:
    description: Boolean indicating whether organization will restrict access to Dashboard
      (including the API) from certain IP addresses.
    type: bool
  enforcePasswordExpiration:
    description: Boolean indicating whether users are forced to change their password
      every X number of days.
    type: bool
  enforceStrongPasswords:
    description: Deprecated. Values of 'false' are always ignored.
    type: bool
  enforceTwoFactorAuth:
    description: Boolean indicating whether users in this organization will be required
      to use an extra verification code when logging in to Dashboard. This code will
      be sent to their mobile phone via SMS, or can be generated by the authenticator
      application.
    type: bool
  idleTimeoutMinutes:
    description: Number of minutes users can remain idle before being logged out of
      their accounts.
    type: int
  loginIpRanges:
    description: List of acceptable IP ranges. Entries can be single IP addresses,
      IP address ranges, and CIDR subnets.
    elements: str
    type: list
  minimumPasswordLength:
    description: Minimum number of characters required in admins' passwords.
    type: int
  numDifferentPasswords:
    description: Number of recent passwords that new password must be distinct from.
    type: int
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  passwordExpirationDays:
    description: Number of days after which users will be forced to change their password.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations updateOrganizationLoginSecurity
    description: Complete reference of the updateOrganizationLoginSecurity API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-login-security
notes:
  - SDK Method used are
    organizations.Organizations.update_organization_login_security,
  - Paths used are
    put /organizations/{organizationId}/loginSecurity,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.organizations_login_security:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    accountLockoutAttempts: 3
    apiAuthentication:
      ipRestrictionsForKeys:
        enabled: true
        ranges:
          - 192.195.83.1
          - 192.168.33.33
    enforceAccountLockout: true
    enforceDifferentPasswords: true
    enforceIdleTimeout: true
    enforceLoginIpRanges: true
    enforcePasswordExpiration: true
    enforceStrongPasswords: true
    enforceTwoFactorAuth: true
    idleTimeoutMinutes: 30
    loginIpRanges:
      - 192.195.83.1
      - 192.195.83.255
    minimumPasswordLength: 12
    numDifferentPasswords: 3
    organizationId: string
    passwordExpirationDays: 90
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enforcePasswordExpiration": true,
      "passwordExpirationDays": 0,
      "enforceDifferentPasswords": true,
      "numDifferentPasswords": 0,
      "enforceStrongPasswords": true,
      "minimumPasswordLength": 0,
      "enforceAccountLockout": true,
      "accountLockoutAttempts": 0,
      "enforceIdleTimeout": true,
      "idleTimeoutMinutes": 0,
      "enforceTwoFactorAuth": true,
      "enforceLoginIpRanges": true,
      "loginIpRanges": [
        "string"
      ],
      "apiAuthentication": {
        "ipRestrictionsForKeys": {
          "enabled": true,
          "ranges": [
            "string"
          ]
        }
      }
    }
"""



#!/usr/bin/python
# Copyright (c) 2017 Red Hat Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: manageiq_alerts

short_description: Configuration of alerts in ManageIQ
extends_documentation_fragment:
  - community.general._manageiq
  - community.general._attributes

author: Elad Alfassa (@elad661) <ealfassa@redhat.com>
description:
  - The manageiq_alerts module supports adding, updating and deleting alerts in ManageIQ.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  state:
    type: str
    description:
      - V(absent) - alert should not exist,
      - V(present) - alert should exist.
    choices: ['absent', 'present']
    default: 'present'
  description:
    type: str
    description:
      - The unique alert description in ManageIQ.
      - Required when state is "absent" or "present".
  resource_type:
    type: str
    description:
      - The entity type for the alert in ManageIQ. Required when O(state=present).
    choices: ['Vm', 'ContainerNode', 'MiqServer', 'Host', 'Storage', 'EmsCluster', 'ExtManagementSystem', 'MiddlewareServer']
  expression_type:
    type: str
    description:
      - Expression type.
    default: hash
    choices: ["hash", "miq"]
  expression:
    type: dict
    description:
      - The alert expression for ManageIQ.
      - Can either be in the "Miq Expression" format or the "Hash Expression format".
      - Required if O(state=present).
  enabled:
    description:
      - Enable or disable the alert. Required if O(state=present).
    type: bool
  options:
    type: dict
    description:
      - Additional alert options, such as notification type and frequency.
"""

EXAMPLES = r"""
- name: Add an alert with a "hash expression" to ManageIQ
  community.general.manageiq_alerts:
    state: present
    description: Test Alert 01
    options:
      notifications:
        email:
          to: ["example@example.com"]
          from: "example@example.com"
    resource_type: ContainerNode
    expression:
      eval_method: hostd_log_threshold
      mode: internal
      options: {}
    enabled: true
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!

- name: Add an alert with a "miq expression" to ManageIQ
  community.general.manageiq_alerts:
    state: present
    description: Test Alert 02
    options:
      notifications:
        email:
          to: ["example@example.com"]
          from: "example@example.com"
    resource_type: Vm
    expression_type: miq
    expression:
      and:
        - CONTAINS:
            tag: Vm.managed-environment
            value: prod
        - not:
          CONTAINS:
            tag: Vm.host.managed-environment
            value: prod
    enabled: true
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!

- name: Delete an alert from ManageIQ
  community.general.manageiq_alerts:
    state: absent
    description: Test Alert 01
    manageiq_connection:
      url: 'http://127.0.0.1:3000'
      username: 'admin'
      password: 'smartvm'
      validate_certs: false # only do this when you trust the network!
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._manageiq import ManageIQ, manageiq_argument_spec


class ManageIQAlert:
    """Represent a ManageIQ alert. Can be initialized with both the format
    we receive from the server and the format we get from the user.
    """

    def __init__(self, alert):
        self.description = alert["description"]
        self.db = alert["db"]
        self.enabled = alert["enabled"]
        self.options = alert["options"]
        self.hash_expression = None
        self.miq_expressipn = None

        if "hash_expression" in alert:
            self.hash_expression = alert["hash_expression"]
        if "miq_expression" in alert:
            self.miq_expression = alert["miq_expression"]
            if "exp" in self.miq_expression:
                # miq_expression is a field that needs a special case, because
                # it is returned surrounded by a dict named exp even though we don't
                # send it with that dict.
                self.miq_expression = self.miq_expression["exp"]

    def __eq__(self, other):
        """Compare two ManageIQAlert objects"""
        return self.__dict__ == other.__dict__


class ManageIQAlerts:
    """Object to execute alert management operations in manageiq."""

    def __init__(self, manageiq):
        self.manageiq = manageiq

        self.module = self.manageiq.module
        self.api_url = self.manageiq.api_url
        self.client = self.manageiq.client
        self.alerts_url = f"{self.api_url}/alert_definitions"

    def get_alerts(self):
        """Get all alerts from ManageIQ"""
        try:
            response = self.client.get(f"{self.alerts_url}?expand=resources")
        except Exception as e:
            self.module.fail_json(msg=f"Failed to query alerts: {e}")
        return response.get("resources", [])

    def validate_hash_expression(self, expression):
        """Validate a 'hash expression' alert definition"""
        # hash expressions must have the following fields
        for key in ["options", "eval_method", "mode"]:
            if key not in expression:
                msg = f"Hash expression is missing required field {key}"
                self.module.fail_json(msg)

    def create_alert_dict(self, params):
        """Create a dict representing an alert"""
        if params["expression_type"] == "hash":
            # hash expression supports depends on https://github.com/ManageIQ/manageiq-api/pull/76
            self.validate_hash_expression(params["expression"])
            expression_type = "hash_expression"
        else:
            # actually miq_expression, but we call it "expression" for backwards-compatibility
            expression_type = "expression"

        # build the alret
        alert = dict(
            description=params["description"],
            db=params["resource_type"],
            options=params["options"],
            enabled=params["enabled"],
        )

        # add the actual expression.
        alert.update({expression_type: params["expression"]})

        return alert

    def add_alert(self, alert):
        """Add a new alert to ManageIQ"""
        try:
            result = self.client.post(self.alerts_url, action="create", resource=alert)

            msg = f"Alert {alert['description']} created successfully: {result}"
            return dict(changed=True, msg=msg)
        except Exception as e:
            description = alert["description"]
            if "Resource expression needs be specified" in str(e):
                # Running on an older version of ManageIQ and trying to create a hash expression
                msg = f"Creating alert {description} failed: Your version of ManageIQ does not support hash_expression"
            else:
                msg = f"Creating alert {description} failed: {e}"
            self.module.fail_json(msg=msg)

    def delete_alert(self, alert):
        """Delete an alert"""
        try:
            result = self.client.post(f"{self.alerts_url}/{alert['id']}", action="delete")
            msg = f"Alert {alert['description']} deleted: {result}"
            return dict(changed=True, msg=msg)
        except Exception as e:
            msg = f"Deleting alert {alert['description']} failed: {e}"
            self.module.fail_json(msg=msg)

    def update_alert(self, existing_alert, new_alert):
        """Update an existing alert with the values from `new_alert`"""
        new_alert_obj = ManageIQAlert(new_alert)
        if new_alert_obj == ManageIQAlert(existing_alert):
            # no change needed - alerts are identical
            return dict(changed=False, msg="No update needed")
        else:
            try:
                url = f"{self.alerts_url}/{existing_alert['id']}"
                result = self.client.post(url, action="edit", resource=new_alert)

                # make sure that the update was indeed successful by comparing
                # the result to the expected result.
                if new_alert_obj == ManageIQAlert(result):
                    # success!
                    msg = f"Alert {existing_alert['description']} updated successfully: {result}"

                    return dict(changed=True, msg=msg)
                else:
                    # unexpected result
                    msg = f"Updating alert {existing_alert['description']} failed, unexpected result {result}"

                    self.module.fail_json(msg=msg)

            except Exception as e:
                description = existing_alert["description"]
                if "Resource expression needs be specified" in str(e):
                    # Running on an older version of ManageIQ and trying to update a hash expression
                    msg = f"Updating alert {description} failed: Your version of ManageIQ does not support hash_expression"
                else:
                    msg = f"Updating alert {description} failed: {e}"
                self.module.fail_json(msg=msg)


def main():
    argument_spec = dict(
        description=dict(type="str"),
        resource_type=dict(
            type="str",
            choices=[
                "Vm",
                "ContainerNode",
                "MiqServer",
                "Host",
                "Storage",
                "EmsCluster",
                "ExtManagementSystem",
                "MiddlewareServer",
            ],
        ),
        expression_type=dict(type="str", default="hash", choices=["miq", "hash"]),
        expression=dict(type="dict"),
        options=dict(type="dict"),
        enabled=dict(type="bool"),
        state=dict(default="present", choices=["present", "absent"]),
    )
    # add the manageiq connection arguments to the arguments
    argument_spec.update(manageiq_argument_spec())

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=[
            ("state", "present", ["description", "resource_type", "expression", "enabled", "options"]),
            ("state", "absent", ["description"]),
        ],
    )

    state = module.params["state"]
    description = module.params["description"]

    manageiq = ManageIQ(module)
    manageiq_alerts = ManageIQAlerts(manageiq)

    existing_alert = manageiq.find_collection_resource_by("alert_definitions", description=description)

    # we need to add or update the alert
    if state == "present":
        alert = manageiq_alerts.create_alert_dict(module.params)

        if not existing_alert:
            # an alert with this description doesn't exist yet, let's create it
            res_args = manageiq_alerts.add_alert(alert)
        else:
            # an alert with this description exists, we might need to update it
            res_args = manageiq_alerts.update_alert(existing_alert, alert)

    # this alert should not exist
    elif state == "absent":
        # if we have an alert with this description, delete it
        if existing_alert:
            res_args = manageiq_alerts.delete_alert(existing_alert)
        else:
            # it doesn't exist, and that's okay
            msg = f"Alert '{description}' does not exist in ManageIQ"
            res_args = dict(changed=False, msg=msg)

    module.exit_json(**res_args)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_online_deployment
version_added: "3.17.0"
short_description: Create, Delete for an Azure Machine Learning Online Deployment
description:
    - Create, Delete for an Azure Machine Learning Online Deployment.
options:
    name:
        description:
            - Name of the Azure ML online_deployment.
        type: str
        required: false
    endpoint_name:
        description:
            - Name of the online endpoint. If not specified here will need to
              be referenced in the I(resource_definition).
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    resource_definition:
        description:
            - Online Deployment definition in YAML
        type: str
        required: false
    description:
        description:
            - Description of the online_deployment.
        type: str
    local:
        description:
            - Update local deployment.
        type: bool
        default: false
    local_enable_gpu:
        description:
            - Enable GPU for local deployment.
        type: bool
        default: false
    package_model:
        description:
            - Create packaged environment from the deployment yaml
              and use the packaged environment for the deployment.
        type: bool
        default: false
    all_traffic:
        description:
            - Sets endpoint traffic 100% to this deployment
              after successful creation.
        type: bool
        default: false
    state:
        description:
            - State of the Online Deployment. Use C(present) to create
              or update.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Online Deployment
  azure.azcollection.azure_rm_ml_online_deployment:
    endpoint_name: endpoint-command-ansible
    resource_group: xxxxx-ml-online_deployment
    ml_workspace: workspace-xxxxxxxxxx
    resource_definition: |
      $schema: https://azuremlschemas.azureedge.net/latest/managedOnlineDeployment.schema.json
      name: {{ online_deployment_name }}
      model:
        path: ./targets/azure_rm_ml_online_deployment/files/model/
      code_configuration:
        code: ./targets/azure_rm_ml_online_deployment/files/onlinescoring/
        scoring_script: score.py
      environment:
        conda_file: ./targets/azure_rm_ml_online_deployment/files/environment/conda.yaml
        image: mcr.microsoft.com/azureml/openmpi4.1.0-ubuntu22.04:latest
      instance_type: Standard_DS3_v2
      instance_count: 1
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_online_deployment:
    description:
        - Online Deployment that was just created or updated.
    returned: always
    type: dict
    sample: {
        "app_insights_enabled": false,
        "code_configuration": {
            "code": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/codes/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/versions/1",
            "scoring_script": "score.py"
        },
        "creation_context": {
            "created_at": "2026-04-17T20:39:38.864149+00:00",
            "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "last_modified_at": "2026-04-17T20:39:38.864151+00:00"
        },
        "egress_public_network_access": "enabled",
        "endpoint_name": "endpoint-command-ansible",
        "environment": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/environments/CliV2AnonymousEnvironment/versions/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
        "environment_variables": {
            "AML_APP_ROOT": "/var/azureml-app/onlinescoring",
            "AZUREML_ENTRY_SCRIPT": "score.py",
            "AZUREML_MODEL_DIR": "/var/azureml-app/azureml-models/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/1"
        },
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/onlineEndpoints/endpoint-command-ansible/deployments/deployment-command-xxxxxxxxxx",
        "instance_count": 1,
        "instance_type": "Standard_DS3_v2",
        "liveness_probe": {
            "failure_threshold": 30,
            "initial_delay": 10,
            "period": 10,
            "success_threshold": 1,
            "timeout": 2
        },
        "model": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/models/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/versions/1",
        "name": "deployment-command-e76e189747",
        "properties": {
            "AzureAsyncOperationUri": "https://management.azure.com/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.MachineLearningServices/locations/eastus/mfeOperationsStatus/odidp:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx?api-version=2023-04-01-preview"
        },
        "provisioning_state": "Succeeded",
        "readiness_probe": {
            "failure_threshold": 30,
            "initial_delay": 10,
            "period": 10,
            "success_threshold": 1,
            "timeout": 2
        },
        "request_settings": {
            "max_concurrent_requests_per_instance": 1,
            "request_timeout_ms": 5000
        },
        "scale_settings": {
            "type": "default"
        },
        "tags": {},
        "type": "managed"
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_online_deployment
    from azure.ai.ml.constants._common import REGISTRY_URI_FORMAT
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLOnlineDeployment(MLClientCommon):

    UPDATE_OVERRIDES = ['description',
                        'tags',
                        'endpoint_name']

    PARAM_OVERRIDES = UPDATE_OVERRIDES + ['name']

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            endpoint_name=dict(
                type='str',
                required=False,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            local=dict(
                type='bool',
                default=False,
            ),
            local_enable_gpu=dict(
                type='bool',
                default=False,
            ),
            package_model=dict(
                type='bool',
                default=False,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            all_traffic=dict(
                type='bool',
                default=False,
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent'],
            ),
        )

        self._client = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_online_deployment={}
        )

        required_if = [
            ('state', 'present', ['resource_definition']),
            ('resource_definition', None, ['name', 'endpoint_name'])
        ]

        super(AzureRMMLOnlineDeployment, self).__init__(self.module_arg_spec,
                                                        supports_check_mode=True,
                                                        required_if=required_if,
                                                        )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False

        if self.resource_definition:
            resource_definition = io.StringIO(self.resource_definition)
            params_override = self.update_params(kwargs,
                                                 self.UPDATE_OVERRIDES)
            ml_online_deployment = load_online_deployment(
                resource_definition,
                params_override=params_override
            )
            deployment_name = ml_online_deployment.name
            endpoint_name = ml_online_deployment.endpoint_name
        else:
            deployment_name = self.name
            endpoint_name = self.endpoint_name

        ml_online_deployment_info = self.get(deployment_name,
                                             endpoint_name,
                                             local=self.local)

        if self.state == 'present':

            if ml_online_deployment_info:
                # Update
                params_override = self.update_params(kwargs,
                                                     self.UPDATE_OVERRIDES,
                                                     ml_online_deployment_info)

                if self.resource_definition:
                    resource_definition = io.StringIO(self.resource_definition)

                # Generate ml_online_deployment based on ansible args passed in.
                ml_online_deployment = load_online_deployment(
                    resource_definition,
                    params_override=params_override
                )
                ml_online_deployment_info = self.entity_to_dict(ml_online_deployment_info)
                changed = not self.default_compare(
                    {},
                    self.entity_to_dict(ml_online_deployment),
                    ml_online_deployment_info,
                    '',
                    self.results
                )
                if changed and not self.check_mode:
                    response = self.client.begin_create_or_update(
                        ml_online_deployment,
                        local=self.local,
                        local_enable_gpu=self.local_enable_gpu,
                    )
                    ml_online_deployment_object = self.get_poller_result(response)
                    ml_online_deployment_info = self.entity_to_dict(ml_online_deployment_object)
            else:
                # Create
                if isinstance(ml_online_deployment.model, str) and \
                        ml_online_deployment.model.startswith(REGISTRY_URI_FORMAT):
                    self.ml_registry = ml_online_deployment.model.split("/")[3]

                changed = True
                if not self.check_mode:
                    response = self.client.begin_create_or_update(
                        ml_online_deployment,
                        local=self.local,
                        local_enable_gpu=self.local_enable_gpu,
                        package_model=self.package_model
                    )
                    ml_online_deployment = self.get_poller_result(response)
                    ml_online_deployment_info = self.entity_to_dict(ml_online_deployment)
            if not self.check_mode and self.all_traffic:
                self._all_traffic(ml_online_deployment, local=self.local)

        elif self.state == 'absent':
            if ml_online_deployment_info:
                # Delete
                changed = True
                if not self.check_mode:
                    response = self.client.online_deployments.begin_delete(
                        name=deployment_name,
                        endpoint_name=endpoint_name,
                        local=self.local
                    )
                    ml_online_deployment_info = self.get_poller_result(response)

        self.results['ml_online_deployment'] = ml_online_deployment_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides, ml_online_deployment=None):
        params_override = []

        # If ml_online_deployment is defined use those values as a base.
        # Update can't change some of these values but we need
        # to keep the same values as before the update.
        if ml_online_deployment:
            items = self.entity_to_dict(ml_online_deployment).items()
            params_override = [{k: v} for k, v in items]

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()):
            if kwargs[key] is not None and key in param_overrides:
                params_override.append({key: kwargs[key]})
        return params_override

    def get(self, name, endpoint_name, local=None, as_dict=False):
        try:
            # Attempt to retrieve ml_online_deployment based on name
            ml_online_deployment_info = self.client.online_deployments.get(
                name=name,
                endpoint_name=endpoint_name,
                local=local
            )
            if as_dict:
                ml_online_deployment_info = self.entity_to_dict(ml_online_deployment_info)
        except ResourceNotFoundError:
            ml_online_deployment_info = None
        return ml_online_deployment_info

    def _all_traffic(self, ml_deployment, local=False):
        endpoint = self.client.online_endpoints.get(
            ml_deployment.endpoint_name,
            local=local
        )
        endpoint.traffic = {ml_deployment.name: 100}
        response = self.client.begin_create_or_update(endpoint, local=local)
        return self.get_poller_result(response)


def main():
    AzureRMMLOnlineDeployment()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.rulesoperations?view=azure-python

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_afdrules
version_added: "3.4.0"
short_description: Manage an Azure Front Door Rules
description:
    - Create, update and delete an Azure Front Door Rules to be used by a Front Door Service Profile created using azure_rm_cdnprofile.

options:
    actions:
        description:
            - A list of actions that are executed when all the conditions of a rule are satisfied.
        type: list
        elements: dict
        suboptions:
            cache_behavior:
                description:
                    - Caching behavior for the requests.
                type: str
                choices:
                    - BypassCache
                    - Override
                    - SetIfMissing
                    - HonorOrigin
                    - OverrideAlways
                    - OverrideIfOriginMissing
            cache_duration:
                description:
                    - The duration for which the content needs to be cached. Allowed format is [d.]hh:mm:ss.
                type: str
            custom_fragment:
                description:
                    - Fragment to add to the redirect URL. Fragment is the part of the URL that comes after #. Do not include the #
                type: str
            custom_hostname:
                description:
                    - Host to redirect. Leave empty to use the incoming host as the destination host.
                type: str
            custom_path:
                description:
                    - The full path to redirect. Path cannot be empty and must start with /. Leave empty to use the incoming path as destination path.
                type: str
            custom_query_string:
                description:
                    - The set of query strings to be placed in the redirect URL.
                    - Setting this value would replace any existing query string; leave empty to preserve the incoming query string.
                    - Query string must be in <key>=:code:<value> format. ? and & will be added automatically so do not include them.
                type: str
            destination:
                description:
                    - Define the relative URL to which the above requests will be rewritten by.
                type: str
            destination_protocol:
                description:
                    - Protocol to use for the redirect.
                default: MatchRequest
                type: str
                choices:
                    - Http
                    - Https
                    - MatchRequest
            forwarding_protocol:
                description:
                    - Protocol this rule will use when forwarding traffic to backends.
                default: MatchRequest
                type: str
                choices:
                    - Http
                    - Https
                    - MatchRequest
            header_action:
                description:
                    - Action to perform.
                type: str
                choices:
                    - Append
                    - Overwrite
                    - Delete
            header_name:
                description:
                    - Name of the header to modify.
                type: str
            is_compression_enabled:
                description:
                    - The caching configuration for this route. Indicates whether content compression is enabled on AzureFrontDoor.
                    - If compression is enabled, content will be served as compressed if user requests for a compressed version.
                    - Content won't be compressed on AzureFrontDoor when requested content is smaller than 1 byte or larger than 1 MB.
                type: str
                choices:
                    - Enabled
                    - Disabled
            name:
                description:
                    - The name of the action for the delivery rule.
                type: str
                required: true
                choices:
                    - ModifyRequestHeader
                    - ModifyResponseHeader
                    - RouteConfigurationOverride
                    - UrlRedirect
                    - UrlRewrite
            origin_group:
                description:
                    - defines the OriginGroup that would override the DefaultOriginGroup.
                type: str
            preserve_unmatched_path:
                description:
                    - Whether to preserve unmatched path.
                default: True
                type: bool
            query_parameters:
                description:
                    - query parameters to include or exclude (comma separated).
                type: str
            query_string_caching_behavior:
                description:
                    - The caching configuration for this route. Defines how Frontdoor caches requests that include query strings.
                    - You can ignore any query strings when caching, ignore specific query strings, cache every request with a unique URL,
                    - or cache specific query strings.
                type: str
                choices:
                    - IgnoreQueryString
                    - UseQueryString
                    - IgnoreSpecifiedQueryStrings
                    - IncludeSpecifiedQueryStrings
            redirect_type:
                description:
                    - The redirect type the rule will use when redirecting traffic.
                type: str
                choices:
                    - Moved
                    - Found
                    - TemporaryRedirect
                    - PermanentRedirect
            source_pattern:
                description:
                    - Define a request URI pattern that identifies the type of requests that may be rewritten.
                    - If value is blank, all strings are matched.
                type: str
            value:
                description:
                    - Value for the specified action.
                type: str
    conditions:
        description:
            - A list of conditions that must be matched for the actions to be executed.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the condition for the delivery rule.
                type: str
                required: true
                choices:
                    - ClientPort
                    - Cookies
                    - HostName
                    - HttpVersion
                    - IsDevice
                    - PostArgs
                    - QueryString
                    - RemoteAddress
                    - RequestBody
                    - RequestHeader
                    - RequestMethod
                    - RequestScheme
                    - RequestUri
                    - ServerPort
                    - SocketAddr
                    - SslProtocol
                    - UrlFileExtension
                    - UrlFileName
                    - UrlPath
            operator:
                description:
                    - Describes operator to be matched.
                type: str
                required: true
                choices:
                    - Any
                    - Equal
                    - Contains
                    - BeginsWith
                    - EndsWith
                    - LessThan
                    - LessThanOrEqual
                    - GreaterThan
                    - GreaterThanOrEqual
                    - RegEx
                    - IPMatch
                    - GeoMatch
            match_values:
                description:
                    - The match value for the condition of the delivery rule.
                type: list
                elements: str
            negate_condition:
                description:
                    - Describes if this is a negate condition or not.
                type: bool
            selector:
                description:
                    - Name of item to be matched.
                type: str
            transforms:
                description:
                    - List of transforms.
                type: list
                elements: str
                choices:
                    - Lowercase
                    - RemoveNulls
                    - Trim
                    - Uppercase
                    - URLDecode
                    - URLEncode
    match_processing_behavior:
        description:
            - If this rule is a match should the rules engine continue running the remaining rules or stop.
        default: Continue
        type: str
        choices:
            - Continue
            - Stop
    name:
        description:
            - Name of the delivery rule which is unique within the endpoint.
        required: true
        type: str
    order:
        description:
            - The order in which the rules are applied for the endpoint.
            - A rule with a lesser order will be applied before a rule with a greater order. Rule with order 0 is a special rule.
            - It does not require any condition and actions listed in it will always be applied.
        type: int
    profile_name:
        description:
            - Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group.
        required: true
        type: str
    resource_group:
        description:
            - Name of the Resource group within the Azure subscription.
        required: true
        type: str
    rule_set_name:
        description:
            - Name of the rule set under the profile.
        required: true
        type: str
    state:
        description:
            - Assert the state of the CDN profile. Use C(present) to create or update a CDN profile and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''
EXAMPLES = '''
- name: Create an AFD Rule with some actions and conditions
  azure_rm_afdrules:
    name: myRule1
    rule_set_name: myRuleset
    profile_name: myProfile
    resource_group: myResourceGroup
    match_processing_behavior: Continue
    order: 1
    action:
      - name: ModifyResponseHeader
        header_action: Append
        header_name: Content-Security-Policy
        value: "frame-ancestors 'response'"
      - name: RouteConfigurationOverride
        origin_group: "myOtherOriginGroup"
    conditions:
      - name: UrlPath
        match_values:
          - ".auth"
        negate_condition: false
        operator: "BeginsWith"
        transforms:
          - LowerCase
'''
RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    example:  # TODO: Put example value in here
'''
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.cdn.models import Rule, RuleUpdateParameters, ResourceReference

    # Models for Actions:
    from azure.mgmt.cdn.models import DeliveryRuleRequestHeaderAction, HeaderActionParameters  # ModifyRequestHeader
    from azure.mgmt.cdn.models import DeliveryRuleResponseHeaderAction  # ModifyResponseHeader
    from azure.mgmt.cdn.models import DeliveryRuleRouteConfigurationOverrideAction, RouteConfigurationOverrideActionParameters, \
        OriginGroupOverride, ForwardingProtocol, CacheConfiguration  # RouteConfigurationOverride
    from azure.mgmt.cdn.models import UrlRedirectAction, UrlRedirectActionParameters  # UrlRedirect
    from azure.mgmt.cdn.models import UrlRewriteAction, UrlRewriteActionParameters  # UrlRewrite

    # Models for Conditions:
    from azure.mgmt.cdn.models import DeliveryRuleClientPortCondition, ClientPortMatchConditionParameters  # ClientPort
    from azure.mgmt.cdn.models import DeliveryRuleCookiesCondition, CookiesMatchConditionParameters  # Cookies
    from azure.mgmt.cdn.models import DeliveryRuleHostNameCondition, HostNameMatchConditionParameters  # HostName
    from azure.mgmt.cdn.models import DeliveryRuleHttpVersionCondition, HttpVersionMatchConditionParameters  # HttpVersion
    from azure.mgmt.cdn.models import DeliveryRuleIsDeviceCondition, IsDeviceMatchConditionParameters  # IsDevice
    from azure.mgmt.cdn.models import DeliveryRulePostArgsCondition, PostArgsMatchConditionParameters  # PostArgs
    from azure.mgmt.cdn.models import DeliveryRuleQueryStringCondition, QueryStringMatchConditionParameters  # QueryString
    from azure.mgmt.cdn.models import DeliveryRuleRemoteAddressCondition, RemoteAddressMatchConditionParameters  # RemoteAddress
    from azure.mgmt.cdn.models import DeliveryRuleRequestBodyCondition, RequestBodyMatchConditionParameters  # RequestBody
    from azure.mgmt.cdn.models import DeliveryRuleRequestHeaderCondition, RequestHeaderMatchConditionParameters  # RequestHeader
    from azure.mgmt.cdn.models import DeliveryRuleRequestMethodCondition, RequestMethodMatchConditionParameters  # RequestMethod
    from azure.mgmt.cdn.models import DeliveryRuleRequestSchemeCondition, RequestSchemeMatchConditionParameters  # RequestScheme
    from azure.mgmt.cdn.models import DeliveryRuleRequestUriCondition, RequestUriMatchConditionParameters  # RequestUri
    from azure.mgmt.cdn.models import DeliveryRuleServerPortCondition, ServerPortMatchConditionParameters  # ServerPort
    from azure.mgmt.cdn.models import DeliveryRuleSocketAddrCondition, SocketAddrMatchConditionParameters  # SocketAddr
    from azure.mgmt.cdn.models import DeliveryRuleSslProtocolCondition, SslProtocolMatchConditionParameters  # SslProtocol
    from azure.mgmt.cdn.models import DeliveryRuleUrlFileExtensionCondition, UrlFileExtensionMatchConditionParameters  # UrlFileExtension
    from azure.mgmt.cdn.models import DeliveryRuleUrlFileNameCondition, UrlFileNameMatchConditionParameters  # UrlFileName
    from azure.mgmt.cdn.models import DeliveryRuleUrlPathCondition, UrlPathMatchConditionParameters  # UrlPath

    from azure.core.serialization import NULL as AzureCoreNull
except ImportError as ec:
    # This is handled in azure_rm_common
    pass


class AzureRMRules(AzureRMModuleBase):
    ''' Class for Azure Front Door Rules '''

    def __init__(self):
        self.module_arg_spec = dict(
            actions=dict(
                type='list',
                elements='dict',
                options=dict(
                    cache_behavior=dict(
                        type='str',
                        choices=[
                            'BypassCache',
                            'Override',
                            'SetIfMissing',
                            'HonorOrigin',
                            'OverrideAlways',
                            'OverrideIfOriginMissing'
                        ]
                    ),
                    cache_duration=dict(type='str'),
                    custom_fragment=dict(type='str'),
                    custom_hostname=dict(type='str'),
                    custom_path=dict(type='str'),
                    custom_query_string=dict(type='str'),
                    destination=dict(type='str'),
                    destination_protocol=dict(
                        type='str',
                        default='MatchRequest',
                        choices=['Http', 'Https', 'MatchRequest']
                    ),
                    forwarding_protocol=dict(
                        type='str',
                        default="MatchRequest",
                        choices=['Http', 'Https', 'MatchRequest']
                    ),
                    header_action=dict(
                        type='str',
                        choices=['Append', 'Overwrite', 'Delete']
                    ),
                    header_name=dict(type='str'),
                    is_compression_enabled=dict(
                        type='str',
                        choices=['Enabled', 'Disabled']
                    ),
                    name=dict(
                        type='str',
                        required=True,
                        choices=[
                            'ModifyRequestHeader',
                            'ModifyResponseHeader',
                            'RouteConfigurationOverride',
                            'UrlRedirect',
                            'UrlRewrite'
                        ]
                    ),
                    origin_group=dict(type='str'),
                    preserve_unmatched_path=dict(type='bool', default=True),
                    query_parameters=dict(type='str'),
                    query_string_caching_behavior=dict(
                        type='str',
                        choices=[
                            "IgnoreQueryString",
                            "UseQueryString",
                            "IgnoreSpecifiedQueryStrings",
                            "IncludeSpecifiedQueryStrings"
                        ]
                    ),
                    redirect_type=dict(
                        type='str',
                        choices=[
                            "Moved",
                            "Found",
                            "TemporaryRedirect",
                            "PermanentRedirect"
                        ]
                    ),
                    source_pattern=dict(type='str'),
                    value=dict(type='str')
                )
            ),
            conditions=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(
                        type='str',
                        required=True,
                        choices=[
                            "ClientPort",
                            "Cookies",
                            "HostName",
                            "HttpVersion",
                            "IsDevice",
                            "PostArgs",
                            "QueryString",
                            "RemoteAddress",
                            "RequestBody",
                            "RequestHeader",
                            "RequestMethod",
                            "RequestScheme",
                            "RequestUri",
                            "ServerPort",
                            "SocketAddr",
                            "SslProtocol",
                            "UrlFileExtension",
                            "UrlFileName",
                            "UrlPath"
                        ]
                    ),
                    operator=dict(
                        type='str',
                        required=True,
                        choices=[
                            'Any',
                            'Equal',
                            'Contains',
                            'BeginsWith',
                            'EndsWith',
                            'LessThan',
                            'LessThanOrEqual',
                            'GreaterThan',
                            'GreaterThanOrEqual',
                            'RegEx',
                            'IPMatch',
                            'GeoMatch'
                        ]
                    ),
                    match_values=dict(type='list', elements='str'),
                    negate_condition=dict(type='bool'),
                    selector=dict(type='str'),
                    transforms=dict(
                        type='list',
                        elements='str',
                        choices=[
                            'Lowercase',
                            'RemoveNulls',
                            'Trim',
                            'Uppercase',
                            'URLDecode',
                            'URLEncode'
                        ]
                    )
                )
            ),
            match_processing_behavior=dict(
                type='str',
                default='Continue',
                choices=['Continue', 'Stop']
            ),
            name=dict(type='str', required=True),
            order=dict(type='int'),
            profile_name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            rule_set_name=dict(type='str', required=True),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )
        self.actions = None
        self.conditions = None
        self.match_processing_behavior = None
        self.rule_set_name = None

        self.name = None
        self.order = None
        self.profile_name = None
        self.resource_group = None
        self.state = None

        self.response = None
        self.parameters = None

        self.results = dict(changed=False)

        super(AzureRMRules, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=False)

    def check_for_field(self, param_list, item, field):
        ''' Utility to check for a field and fail if not found '''
        if field not in param_list.keys():
            self.fail('Required field {1} missing for {0}'.format(item, field))
        elif param_list[field] is None:
            self.fail('Required field {1} missing for {0}'.format(item, field))
        return

    def check_required_fields(self):
        ''' A slightly more complex parameters check for Actions and Conditions '''

        if self.actions:
            required_fields_actions = {
                'ModifyRequestHeader': ['header_action', 'header_name'],
                'ModifyResponseHeader': ['header_action', 'header_name'],
                'RouteConfigurationOverride': [],
                'UrlRedirect': ['redirect_type', 'custom_path'],
                'UrlRewrite': ['source_pattern', 'destination']
            }
            for action in self.actions:
                if 'name' not in action.keys():
                    self.fail("Required field 'Name' missing for Action")
                fields = required_fields_actions[action['name']]
                for field in fields:
                    self.check_for_field(action, action['name'], field)

            if len(self.actions) > 10:
                self.fail("Error: You cannot have more than 10 Actions in a single Rule.")

        if self.conditions:
            for condition in self.conditions:
                if 'name' not in condition.keys():
                    self.fail("Required field 'Name' missing for Condition")
                field = "operator"
                self.check_for_field(condition, condition['name'], field)

            if len(self.conditions) > 10:
                self.fail("Error: You cannot have more than 10 Conditions in a single Rule.")

        return False

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self.check_required_fields()

        self.response = self.get_rule()

        if self.state == 'present':

            if not self.response:
                self.log("Need to create the Rule")
                self.build_parameters()

                if not self.check_mode:
                    new_response = self.create_rules()
                    self.results['id'] = new_response['id']

                self.results['changed'] = True

            else:
                self.build_parameters(update=True)
                if self.check_if_rules_are_different():
                    self.update_rules()
                    self.results['changed'] = True

                self.results['id'] = self.response['id']
                self.log('Results : {0}'.format(self.response))

        elif self.state == 'absent':
            if not self.response:
                self.log("Rule {0} does not exist.".format(self.name))
            else:
                self.log("Need to delete the Rule")
                self.results['changed'] = True

                self.results['id'] = self.response['id']
                if not self.check_mode:
                    self.delete_rules()

        return self.results

    def create_rules(self):
        '''
        Creates a Azure Rules.

        :return: deserialized Azure Rules instance state dictionary
        '''
        self.log("Creating the Azure Rules instance {0}".format(self.name))

        try:
            poller = self.cdn_client.rules.begin_create(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.rule_set_name,
                rule_name=self.name,
                rule=self.parameters
            )
            response = self.get_poller_result(poller)
            return rules_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to create Azure Rules instance.')
            self.fail("Error Creating Azure Rules instance: {0}".format(str(exc)))

    def update_rules(self):
        '''
        Update Azure Rules.

        :return: deserialized Azure Rules instance state dictionary
        '''
        self.log("Updating the Azure Rules instance {0}".format(self.name))

        try:
            poller = self.cdn_client.rules.begin_update(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.rule_set_name,
                rule_name=self.name,
                rule_update_properties=self.parameters
            )
            response = self.get_poller_result(poller)
            return rules_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to update Azure Rules instance.')
            self.fail("Error Updating Azure Rules instance: {0}".format(str(exc)))

    def check_if_rules_are_different(self):
        ''' Check if the Rules are different from what exists and what is defined in the yaml '''

        if self.response['match_processing_behavior'] != self.match_processing_behavior:
            return True
        if self.response['order'] != self.order:
            return True

        if len(self.response['actions']) != len(self.actions):
            return True
        for i, item in enumerate(self.response['actions']):
            for field in item:
                if field not in ['type_name']:
                    if field not in ['cache_configuration', 'origin_group_override']:
                        if item[field] != self.actions[i][field]:
                            return True
                    elif field == 'origin_group_override':
                        if (not item[field] and self.actions[i].get("origin_group")) or (item[field] and not self.actions[i].get("origin_group")):
                            return True
                        if item[field] and self.actions[i].get("origin_group"):  # They both exist so check if the values match
                            origin_group_id = item[field].origin_group.id
                            forwarding_protocol = item[field].forwarding_protocol
                            if origin_group_id != self.actions[i].get("origin_group_id"):
                                return True
                            if forwarding_protocol != self.actions[i].get("forwarding_protocol"):
                                return True
                    elif field == 'cache_configuration':
                        if not item[field] and (
                            self.actions[i].get("query_string_caching_behavior") or
                            self.actions[i].get("query_parameters") or
                            self.actions[i].get("is_compression_enabled") or
                            self.actions[i].get("cache_behavior") or
                            self.actions[i].get("cache_duration")
                        ):
                            return True
                        elif item[field] and not (
                            self.actions[i].get("query_string_caching_behavior") or
                            self.actions[i].get("query_parameters") or
                            self.actions[i].get("is_compression_enabled") or
                            self.actions[i].get("cache_behavior") or
                            self.actions[i].get("cache_duration")
                        ):
                            return True
                        elif item[field]:
                            if item[field].query_string_caching_behavior != self.actions[i].get("query_string_caching_behavior"):
                                return True
                            elif item[field].query_parameters != self.actions[i].get("query_parameters"):
                                return True
                            elif item[field].is_compression_enabled != self.actions[i].get("is_compression_enabled"):
                                return True
                            elif item[field].cache_behavior != self.actions[i].get("cache_behavior"):
                                return True
                            elif item[field].cache_duration != self.actions[i].get("cache_duration"):
                                return True
        if len(self.response['conditions']) != len(self.conditions):
            return True
        for i, item in enumerate(self.response['conditions']):
            for field in item:
                if field not in ['type_name']:
                    if isinstance(item[field], list) and len(item[field]) == 0 and not self.conditions[i][field]:
                        pass  # Catch the edge case where we have an empty list and a None object
                    elif item[field] is False and not self.conditions[i][field]:
                        pass  # Catch where a default of False matches the None object
                    elif item[field] != self.conditions[i][field]:
                        return True
        return False

    def build_parameters(self, update=False):
        ''' Build the parameters to be used by either Create or Update '''
        conditions = None
        if self.conditions:
            conditions = []
            for condition in self.conditions:
                conditionrule = None
                if condition.get('name') == 'ClientPort':
                    conditionrule = DeliveryRuleClientPortCondition(
                        parameters=ClientPortMatchConditionParameters(
                            type_name='DeliveryRuleClientPortConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'Cookies':
                    conditionrule = DeliveryRuleCookiesCondition(
                        parameters=CookiesMatchConditionParameters(
                            type_name='DeliveryRuleCookiesConditionParameters',
                            selector=condition.get('selector'),
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'HostName':
                    conditionrule = DeliveryRuleHostNameCondition(
                        parameters=HostNameMatchConditionParameters(
                            type_name='DeliveryRuleHostNameConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'HttpVersion':
                    conditionrule = DeliveryRuleHttpVersionCondition(
                        parameters=HttpVersionMatchConditionParameters(
                            type_name='DeliveryRuleHttpVersionConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'IsDevice':
                    conditionrule = DeliveryRuleIsDeviceCondition(
                        parameters=IsDeviceMatchConditionParameters(
                            type_name='DeliveryRuleIsDeviceConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'PostArgs':
                    conditionrule = DeliveryRulePostArgsCondition(
                        parameters=PostArgsMatchConditionParameters(
                            type_name='DeliveryRulePostArgsConditionParameters',
                            selector=condition.get('selector'),
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'QueryString':
                    conditionrule = DeliveryRuleQueryStringCondition(
                        parameters=QueryStringMatchConditionParameters(
                            type_name='DeliveryRuleQueryStringConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'RemoteAddress':
                    conditionrule = DeliveryRuleRemoteAddressCondition(
                        parameters=RemoteAddressMatchConditionParameters(
                            type_name='DeliveryRuleRemoteAddressConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'RequestBody':
                    conditionrule = DeliveryRuleRequestBodyCondition(
                        parameters=RequestBodyMatchConditionParameters(
                            type_name='DeliveryRuleRequestBodyConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'RequestHeader':
                    conditionrule = DeliveryRuleRequestHeaderCondition(
                        parameters=RequestHeaderMatchConditionParameters(
                            type_name='DeliveryRuleRequestHeaderConditionParameters',
                            selector=condition.get('selector'),
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'RequestMethod':
                    conditionrule = DeliveryRuleRequestMethodCondition(
                        parameters=RequestMethodMatchConditionParameters(
                            type_name='DeliveryRuleRequestMethodConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'RequestScheme':
                    conditionrule = DeliveryRuleRequestSchemeCondition(
                        parameters=RequestSchemeMatchConditionParameters(
                            type_name='DeliveryRuleRequestSchemeConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'RequestUri':
                    conditionrule = DeliveryRuleRequestUriCondition(
                        parameters=RequestUriMatchConditionParameters(
                            type_name='DeliveryRuleRequestUriConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'ServerPort':
                    conditionrule = DeliveryRuleServerPortCondition(
                        parameters=ServerPortMatchConditionParameters(
                            type_name='DeliveryRuleServerPortConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'SocketAddr':
                    conditionrule = DeliveryRuleSocketAddrCondition(
                        parameters=SocketAddrMatchConditionParameters(
                            type_name='DeliveryRuleSocketAddrConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'SslProtocol':
                    conditionrule = DeliveryRuleSslProtocolCondition(
                        parameters=SslProtocolMatchConditionParameters(
                            type_name='DeliveryRuleSslProtocolConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'UrlFileExtension':
                    conditionrule = DeliveryRuleUrlFileExtensionCondition(
                        parameters=UrlFileExtensionMatchConditionParameters(
                            type_name='DeliveryRuleUrlFileExtensionMatchConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'UrlFileName':
                    conditionrule = DeliveryRuleUrlFileNameCondition(
                        parameters=UrlFileNameMatchConditionParameters(
                            type_name='DeliveryRuleUrlFilenameConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                elif condition.get('name') == 'UrlPath':
                    conditionrule = DeliveryRuleUrlPathCondition(
                        parameters=UrlPathMatchConditionParameters(
                            type_name='DeliveryRuleUrlPathMatchConditionParameters',
                            operator=condition.get('operator'),
                            negate_condition=condition.get('negate_condition'),
                            match_values=condition.get('match_values'),
                            transforms=condition.get('transforms')
                        )
                    )
                if conditionrule:
                    conditions.append(conditionrule)

        actions = None
        if self.actions:
            actions = []
            for action in self.actions:
                actionrule = None
                if action.get('name') == 'ModifyRequestHeader':
                    actionrule = DeliveryRuleRequestHeaderAction(
                        parameters=HeaderActionParameters(
                            type_name='DeliveryRuleHeaderActionParameters',
                            header_action=action.get('header_action'),
                            header_name=action.get('header_name'),
                            value=action.get('value')
                        )
                    )
                elif action.get('name') == 'ModifyResponseHeader':
                    actionrule = DeliveryRuleResponseHeaderAction(
                        parameters=HeaderActionParameters(
                            type_name='DeliveryRuleHeaderActionParameters',
                            header_action=action.get('header_action'),
                            header_name=action.get('header_name'),
                            value=action.get('value')
                        )
                    )
                elif action.get('name') == 'RouteConfigurationOverride':
                    origin_group_override = None
                    action['origin_group_id'] = None
                    if action.get('origin_group'):
                        origin_group_id = self.get_origin_group_id(action.get('origin_group'))
                        origin_group_ref = ResourceReference(id=origin_group_id)
                        action['origin_group_id'] = origin_group_id
                        origin_group_override = OriginGroupOverride(
                            origin_group=origin_group_ref,
                            forwarding_protocol=ForwardingProtocol(
                                action.get('forwarding_protocol')
                            )
                        )
                    cache_configuration = AzureCoreNull
                    if action.get('query_string_caching_behavior') or action.get('query_parameters') or \
                            action.get('is_compression_enabled') or action.get('cache_behavior') or action.get('cache_duration'):
                        cache_configuration = CacheConfiguration(
                            query_string_caching_behavior=action.get('query_string_caching_behavior'),
                            query_parameters=action.get('query_parameters'),
                            is_compression_enabled=action.get('is_compression_enabled'),
                            cache_behavior=action.get('cache_behavior'),
                            cache_duration=action.get('cache_duration')
                        )
                    actionrule = DeliveryRuleRouteConfigurationOverrideAction(
                        parameters=RouteConfigurationOverrideActionParameters(
                            type_name="DeliveryRuleRouteConfigurationOverrideActionParameters",
                            origin_group_override=origin_group_override,
                            cache_configuration=cache_configuration
                        )
                    )
                elif action.get('name') == 'UrlRedirect':
                    actionrule = UrlRedirectAction(
                        parameters=UrlRedirectActionParameters(
                            type_name='DeliveryRuleUrlRedirectActionParameters',
                            redirect_type=action.get('redirect_type'),
                            destination_protocol=action.get('destination_protocol'),
                            custom_path=action.get('custom_path'),
                            custom_hostname=action.get('custom_hostname'),
                            custom_query_string=action.get('custom_query_string'),
                            custom_fragment=action.get('custom_fragment')
                        )
                    )
                elif action.get('name') == 'UrlRewrite':
                    actionrule = UrlRewriteAction(
                        parameters=UrlRewriteActionParameters(
                            type_name='DeliveryRuleUrlRewriteActionParameters',
                            source_pattern=action.get('source_pattern'),
                            destination=action.get('destination'),
                            preserve_unmatched_path=action.get('preserve_unmatched_path')
                        )
                    )

                if actionrule:
                    actions.append(actionrule)

        if update:
            self.parameters = RuleUpdateParameters(
                order=self.order,
                conditions=conditions,
                actions=actions,
                match_processing_behavior=self.match_processing_behavior
            )
        else:
            self.parameters = Rule(
                order=self.order,
                conditions=conditions,
                actions=actions,
                match_processing_behavior=self.match_processing_behavior
            )
        return

    def delete_rules(self):
        '''
        Deletes the specified Azure Rules in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Rules {0}".format(self.name))
        try:
            poller = self.cdn_client.rules.begin_delete(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.rule_set_name,
                rule_name=self.name)
            self.get_poller_result(poller)
            return True
        except Exception as exc:
            self.log('Error attempting to delete the Rules.')
            self.fail("Error deleting the Rules: {0}".format(str(exc)))
            return False

    def get_rule(self):
        '''
        Gets the properties of the specified Rules.

        :return: deserialized Rules state dictionary
        '''
        self.log(
            "Checking if the Rules {0} is present".format(self.name))
        try:
            response = self.cdn_client.rules.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.rule_set_name,
                rule_name=self.name
            )
            self.log("Response : {0}".format(response))
            self.log("Rules : {0} found".format(response.name))
            return rules_to_dict(response)
        except Exception as err:
            self.log('Did not find the Rule: {0}'.format(str(err)))
            return False

    def get_origin_group_id(self, origin_group):
        '''
        Gets the ID of the specified Origin Group.

        :return: ID for the Origin Group.
        '''
        self.log(
            "Obtaining ID for Origin Group {0}".format(origin_group))
        try:
            response = self.cdn_client.afd_origin_groups.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                origin_group_name=origin_group)
            self.log("Response : {0}".format(response))
            self.log("Origin Group ID found : {0} found".format(response.id))
            return response.id
        except Exception as err:
            self.fail('Did not find the Origin Group.' + str(err))
            return False


def parse_action_condition(items):
    ''' Convert Actions and Conditions objects to list '''

    parsed = []
    for item in items:
        new_item = {}
        vitem = vars(item)
        for field in vitem:
            if field == 'parameters':
                subvitem = vars(vitem.get('parameters'))
                for subfield in subvitem:
                    if subfield != 'additional_properties':
                        new_item[subfield] = subvitem[subfield]
            elif field != 'additional_properties':
                new_item[field] = vitem[field]
        parsed.append(new_item)
    return parsed


def rules_to_dict(rules):
    ''' Convert Rules object to dictionary '''

    return dict(
        actions=parse_action_condition(rules.actions),
        conditions=parse_action_condition(rules.conditions),
        deployment_status=rules.deployment_status,
        id=rules.id,
        match_processing_behavior=rules.match_processing_behavior,
        name=rules.name,
        order=rules.order,
        provisioning_state=rules.provisioning_state,
        rule_set_name=rules.rule_set_name,
        type=rules.type
    )


def main():
    """Main execution"""
    AzureRMRules()


if __name__ == '__main__':
    main()



# Copyright (c) 2020-2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    get_fingerprint_of_bytes,
    load_publickey,
)
from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover
    from cryptography.hazmat.primitives.asymmetric.types import (  # pragma: no cover
        PublicKeyTypes,
    )

    from ansible_collections.community.crypto.plugins.plugin_utils._action_module import (  # pragma: no cover
        AnsibleActionModule,
    )
    from ansible_collections.community.crypto.plugins.plugin_utils._filter_module import (  # pragma: no cover
        FilterModuleMock,
    )

    GeneralAnsibleModule = t.Union[  # noqa: UP007
        AnsibleModule, AnsibleActionModule, FilterModuleMock
    ]  # pragma: no cover


MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    import cryptography
    import cryptography.hazmat.primitives.asymmetric.ed448
    import cryptography.hazmat.primitives.asymmetric.ed25519
    import cryptography.hazmat.primitives.asymmetric.x448
    import cryptography.hazmat.primitives.asymmetric.x25519
    from cryptography.hazmat.primitives import serialization
except ImportError:
    pass


def _get_cryptography_public_key_info(
    key: PublicKeyTypes,
) -> tuple[str, dict[str, t.Any]]:
    key_public_data: dict[str, t.Any] = {}
    if isinstance(key, cryptography.hazmat.primitives.asymmetric.rsa.RSAPublicKey):
        key_type = "RSA"
        rsa_public_numbers = key.public_numbers()
        key_public_data["size"] = key.key_size
        key_public_data["modulus"] = rsa_public_numbers.n
        key_public_data["exponent"] = rsa_public_numbers.e
    elif isinstance(key, cryptography.hazmat.primitives.asymmetric.dsa.DSAPublicKey):
        key_type = "DSA"
        dsa_parameter_numbers = key.parameters().parameter_numbers()
        dsa_public_numbers = key.public_numbers()
        key_public_data["size"] = key.key_size
        key_public_data["p"] = dsa_parameter_numbers.p
        key_public_data["q"] = dsa_parameter_numbers.q
        key_public_data["g"] = dsa_parameter_numbers.g
        key_public_data["y"] = dsa_public_numbers.y
    elif isinstance(
        key, cryptography.hazmat.primitives.asymmetric.x25519.X25519PublicKey
    ):
        key_type = "X25519"
    elif isinstance(key, cryptography.hazmat.primitives.asymmetric.x448.X448PublicKey):
        key_type = "X448"
    elif isinstance(
        key, cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PublicKey
    ):
        key_type = "Ed25519"
    elif isinstance(
        key, cryptography.hazmat.primitives.asymmetric.ed448.Ed448PublicKey
    ):
        key_type = "Ed448"
    elif isinstance(
        key, cryptography.hazmat.primitives.asymmetric.ec.EllipticCurvePublicKey
    ):
        key_type = "ECC"
        ecc_public_numbers = key.public_numbers()
        key_public_data["curve"] = key.curve.name
        key_public_data["x"] = ecc_public_numbers.x
        key_public_data["y"] = ecc_public_numbers.y
        key_public_data["exponent_size"] = key.curve.key_size
    else:
        key_type = f"unknown ({type(key)})"
    return key_type, key_public_data


class PublicKeyParseError(OpenSSLObjectError):
    def __init__(self, msg: str, *, result: dict[str, t.Any]) -> None:
        super().__init__(msg)
        self.error_message = msg
        self.result = result


class PublicKeyInfoRetrieval:
    def __init__(
        self,
        *,
        module: GeneralAnsibleModule,
        content: bytes | None = None,
        key: PublicKeyTypes | None = None,
    ) -> None:
        # content must be a bytes string
        self.module = module
        self.content = content
        self.key = key

    def _get_public_key(self, binary: bool) -> bytes:
        if self.key is None:
            raise AssertionError("key must be set")  # pragma: no cover
        return self.key.public_bytes(
            serialization.Encoding.DER if binary else serialization.Encoding.PEM,
            serialization.PublicFormat.SubjectPublicKeyInfo,
        )

    def _get_key_info(self) -> tuple[str, dict[str, t.Any]]:
        if self.key is None:
            raise AssertionError("key must be set")  # pragma: no cover
        return _get_cryptography_public_key_info(self.key)

    def get_info(self, *, prefer_one_fingerprint: bool = False) -> dict[str, t.Any]:
        result: dict[str, t.Any] = {}
        if self.key is None:
            try:
                self.key = load_publickey(content=self.content)
            except OpenSSLObjectError as e:
                raise PublicKeyParseError(str(e), result={}) from e

        pk = self._get_public_key(binary=True)
        result["fingerprints"] = (
            get_fingerprint_of_bytes(pk, prefer_one=prefer_one_fingerprint)
            if pk is not None
            else {}
        )

        key_type, key_public_data = self._get_key_info()
        result["type"] = key_type
        result["public_data"] = key_public_data
        return result


def get_publickey_info(
    *,
    module: GeneralAnsibleModule,
    content: bytes | None = None,
    key: PublicKeyTypes | None = None,
    prefer_one_fingerprint: bool = False,
) -> dict[str, t.Any]:
    info = PublicKeyInfoRetrieval(module=module, content=content, key=key)
    return info.get_info(prefer_one_fingerprint=prefer_one_fingerprint)


def select_backend(
    *,
    module: GeneralAnsibleModule,
    content: bytes | None = None,
    key: PublicKeyTypes | None = None,
) -> PublicKeyInfoRetrieval:
    assert_required_cryptography_version(
        module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
    )
    return PublicKeyInfoRetrieval(module=module, content=content, key=key)


__all__ = (
    "PublicKeyInfoRetrieval",
    "PublicKeyParseError",
    "get_publickey_info",
    "select_backend",
)



# -*- coding: utf-8 -*-

# source: https://github.com/tlastowka/calculate_multipart_etag/blob/master/calculate_multipart_etag.py
#
# calculate_multipart_etag  Copyright (C) 2015
#      Tony Lastowka <tlastowka at gmail dot com>
#      https://github.com/tlastowka
#
#
# calculate_multipart_etag is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# calculate_multipart_etag is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with calculate_multipart_etag.  If not, see <http://www.gnu.org/licenses/>.

import hashlib

try:
    from boto3.s3.transfer import TransferConfig

    DEFAULT_CHUNK_SIZE = TransferConfig().multipart_chunksize
except ImportError:
    DEFAULT_CHUNK_SIZE = 5 * 1024 * 1024
    pass  # Handled by AnsibleAWSModule


def calculate_multipart_etag(source_path, chunk_size=DEFAULT_CHUNK_SIZE):
    """
    calculates a multipart upload etag for amazon s3

    Arguments:

    source_path -- The file to calculate the etag for
    chunk_size -- The chunk size to calculate for.
    """

    md5s = []

    with open(source_path, "rb") as fp:
        while True:
            data = fp.read(chunk_size)

            if not data:
                break
            md5 = hashlib.new("md5", usedforsecurity=False)
            md5.update(data)
            md5s.append(md5)

    if len(md5s) == 1:
        new_etag = f'"{md5s[0].hexdigest()}"'
    else:  # > 1
        digests = b"".join(m.digest() for m in md5s)

        new_md5 = hashlib.md5(digests)
        new_etag = f'"{new_md5.hexdigest()}-{len(md5s)}"'

    return new_etag



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_iscsi_static_target_policy_info
short_description: Gather information about iSCSI Static Target Policies in Cisco Intersight
description:
  - Retrieve information about iSCSI Static Target Policies from L(Cisco Intersight,https://intersight.com).
  - Query policies by organization or policy name.
  - Returns structured data with policy metadata and iSCSI target configuration details.
  - If no filters are provided, all iSCSI Static Target Policies will be returned.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization to filter iSCSI Static Target Policies by.
      - Use 'default' for the default organization.
      - When specified, only policies from this organization will be returned.
    type: str
  name:
    description:
      - The exact name of the iSCSI Static Target Policy to retrieve information from.
      - When specified, only the matching policy will be returned.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
# Basic Usage Examples
- name: Fetch all iSCSI Static Target Policies from all organizations
  cisco.intersight.intersight_iscsi_static_target_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
  register: all_iscsi_policies

# Organization-specific Examples
- name: Fetch all iSCSI Static Target Policies from the default organization
  cisco.intersight.intersight_iscsi_static_target_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
  register: default_org_policies

- name: Fetch all iSCSI Static Target Policies from a custom organization
  cisco.intersight.intersight_iscsi_static_target_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Engineering"
  register: engineering_policies

# Specific Policy Examples
- name: Fetch a specific iSCSI Static Target Policy by name
  cisco.intersight.intersight_iscsi_static_target_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "iscsi-static-target-policy-01"
  register: specific_policy

- name: Fetch a specific policy from a specific organization
  cisco.intersight.intersight_iscsi_static_target_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Production"
    name: "iscsi-static-target-policy-prod"
  register: specific_org_policy
'''

RETURN = r'''
api_response:
  description:
    - The API response containing iSCSI Static Target Policy information.
    - Returns a dictionary when querying a single policy or no policies found.
    - Returns a list when multiple policies are found.
  returned: always
  type: dict
  sample:
    # Single policy response (when name parameter is used or only one policy found)
    Name: "iscsi-static-target-policy-01"
    ObjectType: "vnic.IscsiStaticTargetPolicy"
    Moid: "1234567890abcdef12345678"
    Description: "iSCSI static target policy for production servers"
    TargetName: "iqn.1991-05.com.microsoft:winclient1"
    Port: 3260
    Lun:
      LunId: 0
    IscsiIpType: "IPv4"
    IpAddress: "192.168.10.100"
    Organization:
      Name: "default"
      ObjectType: "organization.Organization"
      Moid: "abcdef1234567890abcdef12"
    Tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Owner"
        Value: "Storage-Team"
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)

    # Resource path used to fetch policy info
    resource_path = '/vnic/IscsiStaticTargetPolicies'

    # Get query parameters for policies
    query_params = intersight.set_query_params()

    # Reset api_response before the API call to avoid previous responses
    intersight.result['api_response'] = {}

    # Get iSCSI Static Target policies
    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    policies = intersight.result.get('api_response', [])

    # Create final response structure
    final_api_response = None

    # Ensure policies is always a list for iteration, even if a single dict is returned
    if isinstance(policies, dict):
        policies = [policies]
    elif not isinstance(policies, list):
        policies = []

    # Set final response based on number of policies found
    if len(policies) == 1:
        # Single policy - return as dict
        final_api_response = policies[0]
    elif len(policies) > 1:
        # Multiple policies - return as list
        final_api_response = policies
    else:
        # No policies found - return empty structure
        final_api_response = {}

    # Use intersight.result and update api_response directly
    intersight.result['api_response'] = final_api_response

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2017, Nathan Davison <ndavison85@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: listen_ports_facts
author:
  - Nathan Davison (@ndavison)
description:
  - Gather facts on processes listening on TCP and UDP ports using the C(netstat) or C(ss) commands.
  - This module currently supports Linux only.
requirements:
  - netstat or ss
short_description: Gather facts on processes listening on TCP and UDP ports
notes:
  - C(ss) returns all processes for each listen address and port.
  - This plugin returns each of them, so multiple entries for the same listen address and port are likely in results.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.facts
  - community.general._attributes.facts_module
attributes:
  check_mode:
    version_added: 13.0.0
options:
  command:
    description:
      - Override which command to use for fetching listen ports.
      - By default module uses first found supported command on the system (in alphanumerical order).
    type: str
    choices:
      - netstat
      - ss
    version_added: 4.1.0
  include_non_listening:
    description:
      - Show both listening and non-listening sockets (for TCP this means established connections).
      - Adds the return values RV(ansible_facts.tcp_listen[].state), RV(ansible_facts.udp_listen[].state), RV(ansible_facts.tcp_listen[].foreign_address),
        and RV(ansible_facts.udp_listen[].foreign_address) to the returned facts.
    type: bool
    default: false
    version_added: 5.4.0
"""

EXAMPLES = r"""
- name: Gather facts on listening ports
  community.general.listen_ports_facts:

- name: TCP whitelist violation
  ansible.builtin.debug:
    msg: TCP port {{ item.port }} by pid {{ item.pid }} violates the whitelist
  vars:
    tcp_listen_violations: "{{ ansible_facts.tcp_listen | selectattr('port', 'in', tcp_whitelist) | list }}"
    tcp_whitelist:
      - 22
      - 25
  loop: "{{ tcp_listen_violations }}"

- name: List TCP ports
  ansible.builtin.debug:
    msg: "{{ ansible_facts.tcp_listen | map(attribute='port') | sort | list }}"

- name: List UDP ports
  ansible.builtin.debug:
    msg: "{{ ansible_facts.udp_listen | map(attribute='port') | sort | list }}"

- name: List all ports
  ansible.builtin.debug:
    msg: "{{ (ansible_facts.tcp_listen + ansible_facts.udp_listen) | map(attribute='port') | unique | sort | list }}"

- name: Gather facts on all ports and override which command to use
  community.general.listen_ports_facts:
    command: 'netstat'
    include_non_listening: true
"""

RETURN = r"""
ansible_facts:
  description: Dictionary containing details of TCP and UDP ports with listening servers.
  returned: success
  type: dict
  contains:
    tcp_listen:
      description: A list of processes that are listening on a TCP port.
      returned: success
      type: list
      elements: dict
      contains:
        address:
          description: The address the server is listening on.
          returned: always
          type: str
          sample: "0.0.0.0"
        foreign_address:
          description: The address of the remote end of the socket.
          returned: if O(include_non_listening=true)
          type: str
          sample: "10.80.0.1"
          version_added: 5.4.0
        state:
          description: The state of the socket.
          returned: if O(include_non_listening=true)
          type: str
          sample: "ESTABLISHED"
          version_added: 5.4.0
        name:
          description: The name of the listening process.
          returned: if user permissions allow
          type: str
          sample: "mysqld"
        pid:
          description: The pid of the listening process.
          returned: always
          type: int
          sample: 1223
        port:
          description: The port the server is listening on.
          returned: always
          type: int
          sample: 3306
        protocol:
          description: The network protocol of the server.
          returned: always
          type: str
          sample: "tcp"
        stime:
          description: The start time of the listening process.
          returned: always
          type: str
          sample: "Thu Feb  2 13:29:45 2017"
        user:
          description: The user who is running the listening process.
          returned: always
          type: str
          sample: "mysql"
    udp_listen:
      description: A list of processes that are listening on a UDP port.
      returned: success
      type: list
      elements: dict
      contains:
        address:
          description: The address the server is listening on.
          returned: always
          type: str
          sample: "0.0.0.0"
        foreign_address:
          description: The address of the remote end of the socket.
          returned: if O(include_non_listening=true)
          type: str
          sample: "10.80.0.1"
          version_added: 5.4.0
        state:
          description: The state of the socket. UDP is a connectionless protocol. Shows UCONN or ESTAB.
          returned: if O(include_non_listening=true)
          type: str
          sample: "UCONN"
          version_added: 5.4.0
        name:
          description: The name of the listening process.
          returned: if user permissions allow
          type: str
          sample: "rsyslogd"
        pid:
          description: The pid of the listening process.
          returned: always
          type: int
          sample: 609
        port:
          description: The port the server is listening on.
          returned: always
          type: int
          sample: 514
        protocol:
          description: The network protocol of the server.
          returned: always
          type: str
          sample: "udp"
        stime:
          description: The start time of the listening process.
          returned: always
          type: str
          sample: "Thu Feb  2 13:29:45 2017"
        user:
          description: The user who is running the listening process.
          returned: always
          type: str
          sample: "root"
"""

import platform
import re

from ansible.module_utils.basic import AnsibleModule


def split_pid_name(pid_name):
    """
    Split the entry PID/Program name into the PID (int) and the name (str)
    :param pid_name:  PID/Program String separated with a dash. E.g 51/sshd: returns pid = 51 and name = sshd
    :return: PID (int) and the program name (str)
    """
    try:
        pid, name = pid_name.split("/", 1)
    except ValueError:
        # likely unprivileged user, so add empty name & pid
        return 0, ""
    else:
        name = name.rstrip(":")
        return int(pid), name


def netStatParse(raw):
    """
    The netstat result can be either split in 6,7 or 8 elements depending on the values of state, process and name.
    For UDP the state is always empty. For UDP and TCP the process can be empty.
    So these cases have to be checked.
    :param raw: Netstat raw output String. First line explains the format, each following line contains a connection.
    :return: List of dicts, each dict contains protocol, state, local address, foreign address, port, name, pid for one
     connection.
    """
    results = list()
    for line in raw.splitlines():
        if line.startswith(("tcp", "udp")):
            # set variables to default state, in case they are not specified
            state = ""
            pid_and_name = ""
            process = ""
            formatted_line = line.split()
            protocol, _recv_q, _send_q, address, foreign_address, rest = (
                formatted_line[0],
                formatted_line[1],
                formatted_line[2],
                formatted_line[3],
                formatted_line[4],
                formatted_line[5:],
            )
            address, port = address.rsplit(":", 1)

            if protocol.startswith("tcp"):
                # nestat distinguishes between tcp6 and tcp
                protocol = "tcp"
                if len(rest) == 3:
                    state, pid_and_name, process = rest
                if len(rest) == 2:
                    state, pid_and_name = rest

            if protocol.startswith("udp"):
                # safety measure, similar to tcp6
                protocol = "udp"
                if len(rest) == 2:
                    pid_and_name, process = rest
                if len(rest) == 1:
                    pid_and_name = rest[0]

            pid, name = split_pid_name(pid_name=pid_and_name)
            result = {
                "protocol": protocol,
                "state": state,
                "address": address,
                "foreign_address": foreign_address,
                "port": int(port),
                "name": name,
                "pid": int(pid),
            }
            if result not in results:
                results.append(result)
    return results


def ss_parse(raw):
    """
    The ss_parse result can be either split in 6 or 7 elements depending on the process column,
    e.g. due to unprivileged user.
    :param raw: ss raw output String. First line explains the format, each following line contains a connection.
    :return: List of dicts, each dict contains protocol, state, local address, foreign address, port, name, pid for one
     connection.
    """
    results = list()
    regex_conns = re.compile(pattern=r"\[?(.+?)\]?:([0-9]+)$")
    regex_pid = re.compile(pattern=r'"(.*?)",pid=(\d+)')

    lines = raw.splitlines()

    if len(lines) == 0 or not lines[0].startswith("Netid "):
        # unexpected stdout from ss
        raise OSError(f"Unknown stdout format of `ss`: {raw}")

    # skip headers (-H arg is not present on e.g. Ubuntu 16)
    lines = lines[1:]

    for line in lines:
        cells = line.split(None, 6)
        try:
            if len(cells) == 6:
                # no process column, e.g. due to unprivileged user
                process = ""
                protocol, state, recv_q, send_q, local_addr_port, peer_addr_port = cells
            else:
                protocol, state, recv_q, send_q, local_addr_port, peer_addr_port, process = cells
        except ValueError as e:
            # unexpected stdout from ss
            raise OSError(
                'Expected `ss` table layout "Netid, State, Recv-Q, Send-Q, Local Address:Port, Peer Address:Port" and'
                f'optionally "Process", but got something else: {line}'
            ) from e

        conns = regex_conns.search(local_addr_port)
        pids = regex_pid.findall(process)
        if conns is None and not pids:
            continue

        if not pids:
            # likely unprivileged user, so add empty name & pid
            # as we do in netstat logic to be consistent with output
            pids = [("", 0)]

        address = conns.group(1)
        port = conns.group(2)
        for name, pid in pids:
            result = {
                "protocol": protocol,
                "state": state,
                "address": address,
                "foreign_address": peer_addr_port,
                "port": int(port),
                "name": name,
                "pid": int(pid),
            }
            results.append(result)
    return results


def main():
    command_args = ["-p", "-l", "-u", "-n", "-t"]
    commands_map = {
        "netstat": {"args": [], "parse_func": netStatParse},
        "ss": {"args": [], "parse_func": ss_parse},
    }
    module = AnsibleModule(
        argument_spec=dict(
            command=dict(type="str", choices=list(sorted(commands_map))),
            include_non_listening=dict(default=False, type="bool"),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    if module.params["include_non_listening"]:
        command_args = ["-p", "-u", "-n", "-t", "-a"]

    commands_map["netstat"]["args"] = command_args
    commands_map["ss"]["args"] = command_args

    if platform.system() != "Linux":
        module.fail_json(msg="This module requires Linux.")

    def getPidSTime(pid):
        ps_cmd = module.get_bin_path("ps", True)
        rc, ps_output, stderr = module.run_command([ps_cmd, "-o", "lstart", "-p", str(pid)])
        stime = ""
        if rc == 0:
            for line in ps_output.splitlines():
                if "started" not in line:
                    stime = line
        return stime

    def getPidUser(pid):
        ps_cmd = module.get_bin_path("ps", True)
        rc, ps_output, stderr = module.run_command([ps_cmd, "-o", "user", "-p", str(pid)])
        user = ""
        if rc == 0:
            for line in ps_output.splitlines():
                if line != "USER":
                    user = line
        return user

    result = {
        "changed": False,
        "ansible_facts": {
            "tcp_listen": [],
            "udp_listen": [],
        },
    }

    try:
        command = None
        bin_path = None
        if module.params["command"] is not None:
            command = module.params["command"]
            bin_path = module.get_bin_path(command, required=True)
        else:
            for c in sorted(commands_map):
                bin_path = module.get_bin_path(c, required=False)
                if bin_path is not None:
                    command = c
                    break

        if bin_path is None:
            raise OSError(f"Unable to find any of the supported commands in PATH: {', '.join(sorted(commands_map))}")

        # which ports are listening for connections?
        args = commands_map[command]["args"]
        rc, stdout, stderr = module.run_command([bin_path] + args)
        if rc == 0:
            parse_func = commands_map[command]["parse_func"]
            results = parse_func(stdout)

            for connection in results:
                # only display state and foreign_address for include_non_listening.
                if not module.params["include_non_listening"]:
                    connection.pop("state", None)
                    connection.pop("foreign_address", None)
                connection["stime"] = getPidSTime(connection["pid"])
                connection["user"] = getPidUser(connection["pid"])
                if connection["protocol"].startswith("tcp"):
                    result["ansible_facts"]["tcp_listen"].append(connection)
                elif connection["protocol"].startswith("udp"):
                    result["ansible_facts"]["udp_listen"].append(connection)
    except (KeyError, OSError) as e:
        module.fail_json(msg=f"{e}")

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2018 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: online
author:
  - Remy Leone (@remyleone)
short_description: Scaleway (previously Online SAS or Online.net) inventory source
description:
  - Get inventory hosts from Scaleway (previously Online SAS or Online.net).
options:
  plugin:
    description: Token that ensures this is a source file for the P(community.general.online#inventory) plugin.
    type: string
    required: true
    choices: ['online', 'community.general.online']
  oauth_token:
    required: true
    description: Online OAuth token.
    type: string
    env:
      # in order of precedence
      - name: ONLINE_TOKEN
      - name: ONLINE_API_KEY
      - name: ONLINE_OAUTH_TOKEN
  hostnames:
    description: List of preference about what to use as an hostname.
    type: list
    elements: string
    default:
      - public_ipv4
    choices:
      - public_ipv4
      - private_ipv4
      - hostname
  groups:
    description: List of groups.
    type: list
    elements: string
    choices:
      - location
      - offer
      - rpn
"""

EXAMPLES = r"""
# online_inventory.yml file in YAML format
# Example command line: ansible-inventory --list -i online_inventory.yml

plugin: community.general.online
hostnames:
  - public_ipv4
groups:
  - location
  - offer
  - rpn
"""

import json
from sys import version as python_version
from urllib.parse import urljoin

from ansible.errors import AnsibleError
from ansible.module_utils.ansible_release import __version__ as ansible_version
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.urls import open_url
from ansible.plugins.inventory import BaseInventoryPlugin

from ansible_collections.community.general.plugins.plugin_utils._unsafe import make_unsafe


class InventoryModule(BaseInventoryPlugin):
    NAME = "community.general.online"
    API_ENDPOINT = "https://api.online.net"

    def extract_public_ipv4(self, host_infos):
        try:
            return host_infos["network"]["ip"][0]
        except (KeyError, TypeError, IndexError):
            self.display.warning("An error happened while extracting public IPv4 address. Information skipped.")
            return None

    def extract_private_ipv4(self, host_infos):
        try:
            return host_infos["network"]["private"][0]
        except (KeyError, TypeError, IndexError):
            self.display.warning("An error happened while extracting private IPv4 address. Information skipped.")
            return None

    def extract_os_name(self, host_infos):
        try:
            return host_infos["os"]["name"]
        except (KeyError, TypeError):
            self.display.warning("An error happened while extracting OS name. Information skipped.")
            return None

    def extract_os_version(self, host_infos):
        try:
            return host_infos["os"]["version"]
        except (KeyError, TypeError):
            self.display.warning("An error happened while extracting OS version. Information skipped.")
            return None

    def extract_hostname(self, host_infos):
        try:
            return host_infos["hostname"]
        except (KeyError, TypeError):
            self.display.warning("An error happened while extracting hostname. Information skipped.")
            return None

    def extract_location(self, host_infos):
        try:
            return host_infos["location"]["datacenter"]
        except (KeyError, TypeError):
            self.display.warning("An error happened while extracting datacenter location. Information skipped.")
            return None

    def extract_offer(self, host_infos):
        try:
            return host_infos["offer"]
        except (KeyError, TypeError):
            self.display.warning("An error happened while extracting commercial offer. Information skipped.")
            return None

    def extract_rpn(self, host_infos):
        try:
            return self.rpn_lookup_cache[host_infos["id"]]
        except (KeyError, TypeError):
            self.display.warning("An error happened while extracting RPN information. Information skipped.")
            return None

    def _fetch_information(self, url):
        try:
            response = open_url(url, headers=self.headers)
        except Exception:
            self.display.warning(f"An error happened while fetching: {url}")
            return None

        try:
            raw_data = to_text(response.read(), errors="surrogate_or_strict")
        except UnicodeError as e:
            raise AnsibleError("Incorrect encoding of fetched payload from Online servers") from e

        try:
            return json.loads(raw_data)
        except ValueError as e:
            raise AnsibleError("Incorrect JSON payload") from e

    @staticmethod
    def extract_rpn_lookup_cache(rpn_list):
        lookup = {}
        for rpn in rpn_list:
            for member in rpn["members"]:
                lookup[member["id"]] = rpn["name"]
        return lookup

    def _fill_host_variables(self, hostname, host_infos):
        targeted_attributes = (
            "offer",
            "id",
            "hostname",
            "location",
            "boot_mode",
            "power",
            "last_reboot",
            "anti_ddos",
            "hardware_watch",
            "support",
        )
        for attribute in targeted_attributes:
            self.inventory.set_variable(hostname, attribute, make_unsafe(host_infos[attribute]))

        if self.extract_public_ipv4(host_infos=host_infos):
            self.inventory.set_variable(
                hostname, "public_ipv4", make_unsafe(self.extract_public_ipv4(host_infos=host_infos))
            )
            self.inventory.set_variable(
                hostname, "ansible_host", make_unsafe(self.extract_public_ipv4(host_infos=host_infos))
            )

        if self.extract_private_ipv4(host_infos=host_infos):
            self.inventory.set_variable(
                hostname, "public_ipv4", make_unsafe(self.extract_private_ipv4(host_infos=host_infos))
            )

        if self.extract_os_name(host_infos=host_infos):
            self.inventory.set_variable(hostname, "os_name", make_unsafe(self.extract_os_name(host_infos=host_infos)))

        if self.extract_os_version(host_infos=host_infos):
            self.inventory.set_variable(
                hostname, "os_version", make_unsafe(self.extract_os_name(host_infos=host_infos))
            )

    def _filter_host(self, host_infos, hostname_preferences):
        for pref in hostname_preferences:
            if self.extractors[pref](host_infos):
                return self.extractors[pref](host_infos)

        return None

    def do_server_inventory(self, host_infos, hostname_preferences, group_preferences):
        hostname = self._filter_host(host_infos=host_infos, hostname_preferences=hostname_preferences)

        # No suitable hostname were found in the attributes and the host won't be in the inventory
        if not hostname:
            return

        hostname = make_unsafe(hostname)

        self.inventory.add_host(host=hostname)
        self._fill_host_variables(hostname=hostname, host_infos=host_infos)

        for g in group_preferences:
            group = self.group_extractors[g](host_infos)

            if not group:
                return

            group = make_unsafe(group)

            self.inventory.add_group(group=group)
            self.inventory.add_host(group=group, host=hostname)

    def parse(self, inventory, loader, path, cache=True):
        super().parse(inventory, loader, path)
        self._read_config_data(path=path)

        token = self.get_option("oauth_token")
        hostname_preferences = self.get_option("hostnames")

        group_preferences = self.get_option("groups")
        if group_preferences is None:
            group_preferences = []

        self.extractors = {
            "public_ipv4": self.extract_public_ipv4,
            "private_ipv4": self.extract_private_ipv4,
            "hostname": self.extract_hostname,
        }

        self.group_extractors = {
            "location": self.extract_location,
            "offer": self.extract_offer,
            "rpn": self.extract_rpn,
        }

        self.headers = {
            "Authorization": f"Bearer {token}",
            "User-Agent": f"ansible {ansible_version} Python {python_version.split(' ', 1)[0]}",
            "Content-type": "application/json",
        }

        servers_url = urljoin(InventoryModule.API_ENDPOINT, "api/v1/server")
        servers_api_path = self._fetch_information(url=servers_url)

        if "rpn" in group_preferences:
            rpn_groups_url = urljoin(InventoryModule.API_ENDPOINT, "api/v1/rpn/group")
            rpn_list = self._fetch_information(url=rpn_groups_url)
            self.rpn_lookup_cache = self.extract_rpn_lookup_cache(rpn_list)

        for server_api_path in servers_api_path:
            server_url = urljoin(InventoryModule.API_ENDPOINT, server_api_path)
            raw_server_info = self._fetch_information(url=server_url)

            if raw_server_info is None:
                continue

            self.do_server_inventory(
                host_infos=raw_server_info,
                hostname_preferences=hostname_preferences,
                group_preferences=group_preferences,
            )



#!/usr/bin/python
#
# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_image_build

short_description: Build Docker images using Docker buildx

version_added: 3.6.0

description:
  - This module allows you to build Docker images using Docker's buildx plugin (BuildKit).
  - Note that the module is B(not idempotent) in the sense of classical Ansible modules. The only idempotence check is whether
    the built image already exists. This check can be disabled with the O(rebuild) option.
extends_documentation_fragment:
  - community.docker._docker.cli_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  idempotent:
    support: partial
    details:
      - If O(rebuild=always) the module is not idempotent.

options:
  name:
    description:
      - 'Image name. Name format will be one of: C(name), C(repository/name), C(registry_server:port/name). When pushing or
        pulling an image the name can optionally include the tag by appending C(:tag_name).'
      - Note that image IDs (hashes) and names with digest cannot be used.
    type: str
    required: true
  tag:
    description:
      - Tag for the image name O(name) that is to be tagged.
      - If O(name)'s format is C(name:tag), then the tag value from O(name) will take precedence.
    type: str
    default: latest
  path:
    description:
      - The path for the build environment.
    type: path
    required: true
  dockerfile:
    description:
      - Provide an alternate name for the Dockerfile to use when building an image.
      - This can also include a relative path (relative to O(path)).
    type: str
  cache_from:
    description:
      - List of image names to consider as cache source.
    type: list
    elements: str
  pull:
    description:
      - When building an image downloads any updates to the FROM image in Dockerfile.
    type: bool
    default: false
  network:
    description:
      - The network to use for C(RUN) build instructions.
    type: str
  nocache:
    description:
      - Do not use cache when building an image.
    type: bool
    default: false
  etc_hosts:
    description:
      - Extra hosts to add to C(/etc/hosts) in building containers, as a mapping of hostname to IP address.
      - Instead of an IP address, the special value V(host-gateway) can also be used, which resolves to the host's gateway
        IP and allows building containers to connect to services running on the host.
    type: dict
  args:
    description:
      - Provide a dictionary of C(key:value) build arguments that map to Dockerfile ARG directive.
      - Docker expects the value to be a string. For convenience any non-string values will be converted to strings.
    type: dict
  target:
    description:
      - When building an image specifies an intermediate build stage by name as a final stage for the resulting image.
    type: str
  platform:
    description:
      - Platforms in the format C(os[/arch[/variant]]).
      - Since community.docker 3.10.0 this can be a list of platforms, instead of just a single platform.
    type: list
    elements: str
  shm_size:
    description:
      - Size of C(/dev/shm) in format C(<number>[<unit>]). Number is positive integer. Unit can be V(B) (byte), V(K) (kibibyte,
        1024B), V(M) (mebibyte), V(G) (gibibyte), V(T) (tebibyte), or V(P) (pebibyte).
      - Omitting the unit defaults to bytes. If you omit the size entirely, Docker daemon uses V(64M).
    type: str
  labels:
    description:
      - Dictionary of key value pairs.
    type: dict
  rebuild:
    description:
      - Defines the behavior of the module if the image to build (as specified in O(name) and O(tag)) already exists.
    type: str
    choices:
      - never
      - always
    default: never
  secrets:
    description:
      - Secrets to expose to the build.
    type: list
    elements: dict
    version_added: 3.10.0
    suboptions:
      id:
        description:
          - The secret identifier.
          - The secret will be made available as a file in the container under C(/run/secrets/<id>).
        type: str
        required: true
      type:
        description:
          - Type of the secret.
        type: str
        choices:
          file:
            - Reads the secret from a file on the target.
            - The file must be specified in O(secrets[].src).
          env:
            - Reads the secret from an environment variable on the target.
            - The environment variable must be named in O(secrets[].env).
            - Note that this requires the Buildkit plugin to have version 0.6.0 or newer.
          value:
            - Provides the secret from a given value O(secrets[].value).
            - B(Note) that the secret will be passed as an environment variable to C(docker compose). Use another mean of
              transport if you consider this not safe enough.
            - Note that this requires the Buildkit plugin to have version 0.6.0 or newer.
        required: true
      src:
        description:
          - Source path of the secret.
          - Only supported and required for O(secrets[].type=file).
        type: path
      env:
        description:
          - Environment value of the secret.
          - Only supported and required for O(secrets[].type=env).
        type: str
      value:
        description:
          - Value of the secret.
          - B(Note) that the secret will be passed as an environment variable to C(docker compose). Use another mean of transport
            if you consider this not safe enough.
          - Only supported and required for O(secrets[].type=value).
        type: str
  outputs:
    description:
      - Output destinations.
      - You can provide a list of exporters to export the built image in various places. Note that not all exporters might
        be supported by the build driver used.
      - Note that depending on how this option is used, no image with name O(name) and tag O(tag) might be created, which
        can cause the basic idempotency this module offers to not work.
      - Providing an empty list to this option is equivalent to not specifying it at all. The default behavior is a single
        entry with O(outputs[].type=image).
      - B(Note) that since community.docker 4.2.0, an entry for O(name)/O(tag) is added if O(outputs) has at least one entry
        and no entry has type O(outputs[].type=image) and includes O(name)/O(tag) in O(outputs[].name). This is because the
        module would otherwise pass C(--tag name:image) to the buildx plugin, which for some reason overwrites all images
        in O(outputs) by the C(name:image) provided in O(name)/O(tag).
    type: list
    elements: dict
    version_added: 3.10.0
    suboptions:
      type:
        description:
          - The type of exporter to use.
        type: str
        choices:
          local:
            - This export type writes all result files to a directory on the client. The new files will be owned by the current
              user. On multi-platform builds, all results will be put in subdirectories by their platform.
            - The destination has to be provided in O(outputs[].dest).
          tar:
            - This export type export type writes all result files as a single tarball on the client. On multi-platform builds,
              all results will be put in subdirectories by their platform.
            - The destination has to be provided in O(outputs[].dest).
          oci:
            - This export type writes the result image or manifest list as an L(OCI image layout,
              https://github.com/opencontainers/image-spec/blob/v1.0.1/image-layout.md)
              tarball on the client.
            - The destination has to be provided in O(outputs[].dest).
          docker:
            - This export type writes the single-platform result image as a Docker image specification tarball on the client.
              Tarballs created by this exporter are also OCI compatible.
            - The destination can be provided in O(outputs[].dest). If not specified, the tar will be loaded automatically
              to the local image store.
            - The Docker context where to import the result can be provided in O(outputs[].context).
          image:
            - This exporter writes the build result as an image or a manifest list. When using this driver, the image will
              appear in C(docker images).
            - The image name can be provided in O(outputs[].name). If it is not provided, O(name) and O(tag) will be used.
            - Optionally, image can be automatically pushed to a registry by setting O(outputs[].push=true).
        required: true
      dest:
        description:
          - The destination path.
          - Required for O(outputs[].type=local), O(outputs[].type=tar), O(outputs[].type=oci).
          - Optional for O(outputs[].type=docker).
        type: path
      context:
        description:
          - Name for the Docker context where to import the result.
          - Optional for O(outputs[].type=docker).
        type: str
      name:
        description:
          - Name(s) under which the image is stored under.
          - If not provided, O(name) and O(tag) will be used.
          - Optional for O(outputs[].type=image).
          - This can be a list of strings since community.docker 4.2.0.
        type: list
        elements: str
      push:
        description:
          - Whether to push the built image to a registry.
          - Only used for O(outputs[].type=image).
        type: bool
        default: false
requirements:
  - "Docker CLI with Docker buildx plugin"

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_image_push
  - module: community.docker.docker_image_tag
"""

EXAMPLES = r"""
---
- name: Build Python 3.12 image
  community.docker.docker_image_build:
    name: localhost/python/3.12:latest
    path: /home/user/images/python
    dockerfile: Dockerfile-3.12

- name: Build multi-platform image
  community.docker.docker_image_build:
    name: multi-platform-image
    tag: "1.5.2"
    path: /home/user/images/multi-platform
    platform:
      - linux/amd64
      - linux/arm64/v8
"""

RETURN = r"""
image:
  description: Image inspection results for the affected image.
  returned: success
  type: dict
  sample: {}

command:
  description: The command executed.
  returned: success and for some failures
  type: list
  elements: str
  version_added: 4.2.0
"""

import base64
import os
import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.common.text.formatters import human_to_bytes

from ansible_collections.community.docker.plugins.module_utils._api.utils.utils import (
    parse_repository_tag,
)
from ansible_collections.community.docker.plugins.module_utils._common_cli import (
    AnsibleModuleDockerClient,
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DockerBaseClass,
    clean_dict_booleans_for_docker_api,
    is_image_name_id,
    is_valid_tag,
)
from ansible_collections.community.docker.plugins.module_utils._version import (
    LooseVersion,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


def convert_to_bytes(
    value: str | None,
    module: AnsibleModule,
    name: str,
    unlimited_value: int | None = None,
) -> int | None:
    if value is None:
        return value
    try:
        if unlimited_value is not None and value in ("unlimited", str(unlimited_value)):
            return unlimited_value
        return human_to_bytes(value)
    except ValueError as exc:
        module.fail_json(msg=f"Failed to convert {name} to bytes: {exc}")


def dict_to_list(dictionary: dict[str, t.Any], concat: str = "=") -> list[str]:
    return [f"{k}{concat}{v}" for k, v in sorted(dictionary.items())]


def _quote_csv(text: str) -> str:
    if text.strip() == text and all(i not in text for i in '",\r\n'):
        return text
    text = text.replace('"', '""')
    return f'"{text}"'


class ImageBuilder(DockerBaseClass):
    def __init__(self, client: AnsibleModuleDockerClient) -> None:
        super().__init__()
        self.client = client
        self.check_mode = self.client.check_mode
        parameters = self.client.module.params

        self.cache_from = parameters["cache_from"]
        self.pull = parameters["pull"]
        self.network = parameters["network"]
        self.nocache = parameters["nocache"]
        self.etc_hosts = clean_dict_booleans_for_docker_api(parameters["etc_hosts"])
        self.args = clean_dict_booleans_for_docker_api(parameters["args"])
        self.target = parameters["target"]
        self.platform = parameters["platform"]
        self.shm_size = convert_to_bytes(
            parameters["shm_size"], self.client.module, "shm_size"
        )
        self.labels = clean_dict_booleans_for_docker_api(parameters["labels"])
        self.rebuild = parameters["rebuild"]
        self.secrets = parameters["secrets"]
        self.outputs = parameters["outputs"]

        buildx = self.client.get_client_plugin_info("buildx")
        if buildx is None:
            self.fail(
                f"Docker CLI {self.client.get_cli()} does not have the buildx plugin installed"
            )
        buildx_version = buildx["Version"].lstrip("v")

        if self.secrets:
            for secret in self.secrets:
                if secret["type"] in ("env", "value") and LooseVersion(
                    buildx_version
                ) < LooseVersion("0.6.0"):
                    self.fail(
                        f"The Docker buildx plugin has version {buildx_version}, but 0.6.0 is needed for secrets of type=env and type=value"
                    )
        if (
            self.outputs
            and len(self.outputs) > 1
            and LooseVersion(buildx_version) < LooseVersion("0.13.0")
        ):
            self.fail(
                f"The Docker buildx plugin has version {buildx_version}, but 0.13.0 is needed to specify more than one output"
            )

        self.path = parameters["path"]
        if not os.path.isdir(self.path):
            self.fail(f'"{self.path}" is not an existing directory')
        self.dockerfile = parameters["dockerfile"]
        if self.dockerfile and not os.path.isfile(
            os.path.join(self.path, self.dockerfile)
        ):
            self.fail(
                f'"{os.path.join(self.path, self.dockerfile)}" is not an existing file'
            )

        self.name = parameters["name"]
        self.tag = parameters["tag"]
        if not is_valid_tag(self.tag, allow_empty=True):
            self.fail(f'"{self.tag}" is not a valid docker tag')
        if is_image_name_id(self.name):
            self.fail("Image name must not be a digest")

        # If name contains a tag, it takes precedence over tag parameter.
        repo, repo_tag = parse_repository_tag(self.name)
        if repo_tag:
            self.name = repo
            self.tag = repo_tag

        if is_image_name_id(self.tag):
            self.fail("Image name must not contain a digest, but have a tag")

        if self.outputs:
            found = False
            name_tag = f"{self.name}:{self.tag}"
            for output in self.outputs:
                if output["type"] == "image":
                    if not output["name"]:
                        # Since we no longer pass --tag if --output is provided, we need to set this manually
                        output["name"] = [name_tag]
                    if output["name"] and name_tag in output["name"]:
                        found = True
            if not found:
                self.outputs.append(
                    {
                        "type": "image",
                        "name": [name_tag],
                        "push": False,
                    }
                )
                if LooseVersion(buildx_version) < LooseVersion("0.13.0"):
                    self.fail(
                        f"The output does not include an image with name {name_tag}, and the Docker"
                        f" buildx plugin has version {buildx_version} which only supports one output."
                    )

    def fail(self, msg: str, **kwargs: t.Any) -> t.NoReturn:
        self.client.fail(msg, **kwargs)

    def add_list_arg(self, args: list[str], option: str, values: list[str]) -> None:
        for value in values:
            args.extend([option, value])

    def add_args(self, args: list[str]) -> dict[str, t.Any]:
        environ_update = {}
        if not self.outputs:
            args.extend(["--tag", f"{self.name}:{self.tag}"])
        if self.dockerfile:
            args.extend(["--file", os.path.join(self.path, self.dockerfile)])
        if self.cache_from:
            self.add_list_arg(args, "--cache-from", self.cache_from)
        if self.pull:
            args.append("--pull")
        if self.network:
            args.extend(["--network", self.network])
        if self.nocache:
            args.append("--no-cache")
        if self.etc_hosts:
            self.add_list_arg(args, "--add-host", dict_to_list(self.etc_hosts, ":"))
        if self.args:
            self.add_list_arg(args, "--build-arg", dict_to_list(self.args))
        if self.target:
            args.extend(["--target", self.target])
        if self.platform:
            for platform in self.platform:
                args.extend(["--platform", platform])
        if self.shm_size:
            args.extend(["--shm-size", str(self.shm_size)])
        if self.labels:
            self.add_list_arg(args, "--label", dict_to_list(self.labels))
        if self.secrets:
            random_prefix = None
            for index, secret in enumerate(self.secrets):
                sid = secret["id"]
                if secret["type"] == "file":
                    src = secret["src"]
                    args.extend(["--secret", f"id={sid},type=file,src={src}"])
                if secret["type"] == "env":
                    env = secret["src"]
                    args.extend(["--secret", f"id={sid},type=env,env={env}"])
                if secret["type"] == "value":
                    # We pass values on using environment variables. The user has been warned in the documentation
                    # that they should only use this mechanism when being comfortable with it.
                    if random_prefix is None:
                        # Use /dev/urandom to generate some entropy to make the environment variable's name unguessable
                        random_prefix = (
                            base64.b64encode(os.urandom(16))
                            .decode("utf-8")
                            .replace("=", "")
                        )
                    env_name = (
                        f"ANSIBLE_DOCKER_COMPOSE_ENV_SECRET_{random_prefix}_{index}"
                    )
                    environ_update[env_name] = secret["value"]
                    args.extend(["--secret", f"id={sid},type=env,env={env_name}"])
        if self.outputs:
            for output in self.outputs:
                subargs = []
                if output["type"] == "local":
                    dest = output["dest"]
                    subargs.extend(["type=local", f"dest={dest}"])
                if output["type"] == "tar":
                    dest = output["dest"]
                    subargs.extend(["type=tar", f"dest={dest}"])
                if output["type"] == "oci":
                    dest = output["dest"]
                    subargs.extend(["type=oci", f"dest={dest}"])
                if output["type"] == "docker":
                    subargs.append("type=docker")
                    dest = output["dest"]
                    if output["dest"] is not None:
                        subargs.append(f"dest={dest}")
                    if output["context"] is not None:
                        context = output["context"]
                        subargs.append(f"context={context}")
                if output["type"] == "image":
                    subargs.append("type=image")
                    if output["name"] is not None:
                        name = ",".join(output["name"])
                        subargs.append(f"name={name}")
                    if output["push"]:
                        subargs.append("push=true")
                if subargs:
                    args.extend(
                        ["--output", ",".join(_quote_csv(subarg) for subarg in subargs)]
                    )
        return environ_update

    def build_image(self) -> dict[str, t.Any]:
        image = self.client.find_image(self.name, self.tag)
        results: dict[str, t.Any] = {
            "changed": False,
            "actions": [],
            "image": image or {},
        }

        if image and self.rebuild == "never":
            return results

        results["changed"] = True
        if not self.check_mode:
            args = ["buildx", "build", "--progress", "plain"]
            environ_update = self.add_args(args)
            args.extend(["--", self.path])
            rc, stdout, stderr = self.client.call_cli(
                *args, environ_update=environ_update
            )
            if rc != 0:
                self.fail(
                    f"Building {self.name}:{self.tag} failed",
                    stdout=to_text(stdout),
                    stderr=to_text(stderr),
                    command=args,
                )
            results["stdout"] = to_text(stdout)
            results["stderr"] = to_text(stderr)
            results["image"] = self.client.find_image(self.name, self.tag) or {}
            results["command"] = args

        return results


def main() -> None:
    argument_spec = {
        "name": {"type": "str", "required": True},
        "tag": {"type": "str", "default": "latest"},
        "path": {"type": "path", "required": True},
        "dockerfile": {"type": "str"},
        "cache_from": {"type": "list", "elements": "str"},
        "pull": {"type": "bool", "default": False},
        "network": {"type": "str"},
        "nocache": {"type": "bool", "default": False},
        "etc_hosts": {"type": "dict"},
        "args": {"type": "dict"},
        "target": {"type": "str"},
        "platform": {"type": "list", "elements": "str"},
        "shm_size": {"type": "str"},
        "labels": {"type": "dict"},
        "rebuild": {"type": "str", "choices": ["never", "always"], "default": "never"},
        "secrets": {
            "type": "list",
            "elements": "dict",
            "options": {
                "id": {"type": "str", "required": True},
                "type": {
                    "type": "str",
                    "choices": ["file", "env", "value"],
                    "required": True,
                },
                "src": {"type": "path"},
                "env": {"type": "str"},
                "value": {"type": "str", "no_log": True},
            },
            "required_if": [
                ("type", "file", ["src"]),
                ("type", "env", ["env"]),
                ("type", "value", ["value"]),
            ],
            "mutually_exclusive": [
                ("src", "env", "value"),
            ],
            "no_log": False,
        },
        "outputs": {
            "type": "list",
            "elements": "dict",
            "options": {
                "type": {
                    "type": "str",
                    "choices": ["local", "tar", "oci", "docker", "image"],
                    "required": True,
                },
                "dest": {"type": "path"},
                "context": {"type": "str"},
                "name": {"type": "list", "elements": "str"},
                "push": {"type": "bool", "default": False},
            },
            "required_if": [
                ("type", "local", ["dest"]),
                ("type", "tar", ["dest"]),
                ("type", "oci", ["dest"]),
            ],
            "mutually_exclusive": [
                ("dest", "name"),
                ("dest", "push"),
                ("context", "name"),
                ("context", "push"),
            ],
        },
    }

    client = AnsibleModuleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        needs_api_version=False,
    )

    try:
        results = ImageBuilder(client).build_image()
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_rollback
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Set a checkpoint or rollback to a checkpoint.
description:
- This module offers the ability to set a configuration checkpoint file or rollback
  to a configuration checkpoint file on Cisco NXOS switches.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- Sometimes C(transport=nxapi) may cause a timeout error.
options:
  checkpoint_file:
    description:
    - Name of checkpoint file to create. Mutually exclusive with rollback_to.
    type: str
  rollback_to:
    description:
    - Name of checkpoint file to rollback to. Mutually exclusive with checkpoint_file.
    type:  str
"""

EXAMPLES = """
- cisco.nxos.nxos_rollback:
    checkpoint_file: backup.cfg
    username: '{{ un }}'
    password: '{{ pwd }}'
    host: '{{ inventory_hostname }}'
- cisco.nxos.nxos_rollback:
    rollback_to: backup.cfg
    username: '{{ un }}'
    password: '{{ pwd }}'
    host: '{{ inventory_hostname }}'
"""

RETURN = """
filename:
    description: The filename of the checkpoint/rollback file.
    returned: success
    type: str
    sample: 'backup.cfg'
status:
    description: Which operation took place and whether it was successful.
    returned: success
    type: str
    sample: 'rollback executed'
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import run_commands


def checkpoint(filename, module):
    commands = [
        {"command": "terminal dont-ask", "output": "text"},
        {"command": "checkpoint file %s" % filename, "output": "text"},
    ]
    run_commands(module, commands)


def rollback(filename, module):
    commands = [
        {
            "command": "rollback running-config file %s" % filename,
            "output": "text",
        },
    ]
    run_commands(module, commands)


def main():
    argument_spec = dict(checkpoint_file=dict(required=False), rollback_to=dict(required=False))

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["checkpoint_file", "rollback_to"]],
        supports_check_mode=False,
    )

    checkpoint_file = module.params["checkpoint_file"]
    rollback_to = module.params["rollback_to"]

    status = None
    filename = None
    changed = False

    if checkpoint_file:
        checkpoint(checkpoint_file, module)
        status = "checkpoint file created"
    elif rollback_to:
        rollback(rollback_to, module)
        status = "rollback executed"
    changed = True
    filename = rollback_to or checkpoint_file

    module.exit_json(changed=changed, status=status, filename=filename)


if __name__ == "__main__":
    main()
